/****************************************************************************
 *
 *  cibcomm.h                                       { Ver 0.81 12/09/96 }
 *
 ****************************************************************************/

#ifdef WIN16  
#include <windows.h>
typedef BOOL FAR *LPBOOL;
#define DLLEXPORT far _export pascal
#endif 

#ifdef WIN32
#include <windows.h>
#define DLLEXPORT __stdcall
#endif   

#include "cibtypes.h"
#include "adapter.h"

#define CIB_FC      0x16
#define MAX_CIB_NUM 16
#define FIFO_LEN    8

#define TRUE  1
#define FALSE 0

#pragma pack(1)

/*
 * For the current CIB implementation, the maximum outbound or inbound
 * message size is 128 bytes, and the header (FIFO) data is always 8 bytes
 * long for all transactions.
 */
#define SEGSIZE    128    /* Max seg size that we can send to the device */
#define HEADERSIZE 8      /* Size of a comm layer header packet */

/*
 * Communications Layer OWMS Status Codes -
 */
#define CE_RESET         0
#define CE_MSGINCOMP     1
#define CE_BADSEQ        2 
#define CE_BUFOVERRUN    3
#define CE_BADCKSUM      4
#define CE_HDRSIZE       5
#define CE_DATASIZE      6
#define CE_BADCRC        7
#define CE_DURATION      8
#define CE_FIFONOTEMPTY  9
#define CE_STANDBY       10
#define CE_RESPONSERDY   11
#define CE_RESPINCOMP    12
#define CE_NOHEADER      13
#define CE_MESSAGEOK     15
#define CE_FIRSTBIRTH    29
#define CE_CIREEXEC      30
#define CE_CMDINCOMP     31

#define CE_BPOR          0x40   /* Buferred POR bit in OWMS status byte */

/*
 * Function Error Return Codes
 */
#define ER_NOROOM    2       /* No room in outbound FIFO for a message header */
#define ER_POR       3       /* Constant POR status from device */
#define ER_ACCESS    4       /* Error trying to access device */
#define ER_COLLIDE   5       /* One-Wire Collision Error */
#define ER_ACK       6       /* Failure to get device acknowledge bit */
#define ER_CRC       7       /* CRC test failed */
#define ER_CHECKSUM  8       /* Checksum test failed */
#define ER_NOHEADER  9       /* No header found (no message to read) */
#define ER_BADHEADER 10      /* Bad header form or length */
#define ER_HDRCRC    11      /* Header crc does not match data segment crc */
#define ER_SEQUENCE  12      /* Segments arrived out of sequence or missing */
#define ER_OVERRUN   13      /* Inbound message overflowed buffer limit */
#define ER_COPRUN    14      /* Co-Processor Will Not Complete */
#define ER_RERUN     15      /* Prior command will not complete */
#define ER_ADAPTER   16      /* Bad adapter selection  */

/*
 * Error Group Codes
 */
#define ER_SENDDATA   0x10   /* Error in sending segment data */
#define ER_SENDHDR    0x20   /* Error in sending a segment header */
#define ER_GETSTATUS  0X30   /* Error in geting status */
#define ER_SENDSTATUS 0x40   /* Error in sending status command */
#define ER_SENDRUN    0x50   /* Error in sending run command */
#define ER_SENDRESET  0x60   /* Error in sending reset command */
#define ER_SENDINT    0x70   /* Error in sending interrupt command */
#define ER_RECVHDR    0x80   /* Error in unload of segment header */
#define ER_RECVDATA   0x90   /* Error in unload of segment data */
#define ER_COMMERROR  0xA0   /* Comm Error response from device */

/*
 * One-Wire Command Codes -
 */
#define OW_MICRORUN       0x87
#define OW_MICROINTERRUPT 0x77
#define OW_MICRORESET     0xDD
#define OW_STATUSREAD     0xE1
#define OW_STATUSWRITE    0xD2
#define OW_IPRREAD        0xAA
#define OW_IPRWRITE       0x0F
#define OW_FIFOREAD       0x22
#define OW_FIFOWRITE      0x2D

/*
 * CiB One-Wire Release sequence values -
 */
#define RS_MICRORUN       0x5D73
#define RS_MICROINTERRUPT 0x6D43
#define RS_MICRORESET     0x92BC
#define RS_STATUSWRITE    0x517F
#define RS_FIFOREAD       0x624C
#define RS_FIFOWRITE      0x9DB3

typedef struct _FIFORDWR
{
   BYTE Cmd;
   BYTE Len;
   BYTE FIFOBuf[8];
}
FIFORDWR, *PFIFORDWR, FAR *LPFIFORDWR, NEAR *NPFIFORDWR;

#define LSB(a) (BYTE) ((a) & 0xFF)
#define MSB(a) (BYTE) (((a) & 0xFF00) >> 8)
  

/*
 * Internal Function Prototypes
 */
BYTE SendSegment(LPBYTE, LPBYTE, WORD, LPWORD);
BYTE SendData(LPBYTE, BYTE, LPWORD);
BYTE SendHeader(BYTE, LPBYTE, WORD, BYTE, LPWORD);
BYTE GetStatus(LPBYTE, LPBYTE, LPBYTE, LPBYTE);
BYTE SendStatus(BYTE);
BYTE SendRun(void);
BYTE SendReset(void);
BYTE SendInterrupt(void);
WORD CRC16(BYTE, WORD);
WORD CRC16Buf(LPBYTE, WORD, WORD);
BYTE ReleaseSequence(WORD);
BYTE CheckCRC(WORD);
BYTE CheckStreamCRC(LPBYTE,int,WORD);
BYTE RecvHeader(LPBYTE, LPWORD, LPWORD, LPBYTE, LPWORD);
BYTE RecvData(LPBYTE, BYTE, WORD, LPWORD,BYTE);
void TimeDelay(BYTE);
void GenCRC16Table(void);

/*
 * Access system function prototypes 
 */

#ifndef WIN16 

BYTE iBFirst(void);
BYTE iBNext(void);
BYTE iBDataByte(BYTE);
BYTE iBSetup(BYTE);
BYTE iBAccess(void);
BYTE iBGndtest(void);
BYTE *iBROMData(void);
BYTE iBKeyOpen(void);
BYTE iBKeyClose(void);
BYTE iBDOWCheck(void);
BYTE iBDataBit(BYTE);
void iBDataBlock(LPBYTE, BYTE);
BYTE iBStream(LPBYTE, BYTE);
BYTE iBFastAccess(void);
BYTE iBOverdriveOn(void);
void iBOverdriveOff(void);
void FastSleep(DWORD);
BYTE SetAdapterType(BYTE, char *);
BYTE SetAdapterSpeed(DWORD);
BYTE Adapter5VPrime(void);
BYTE Adapter5VCancel(void);
BYTE SetAdapter5VTime(BYTE);

//uchar SetAdapterType(uchar, char *);
//uchar SetAdapterSpeed(ulong);
//uchar Adapter5VPrime(void);
//uchar Adapter5VCancel(void);
//uchar SetAdapter5VTime(uchar);

#else

BYTE FAR PASCAL iBFirst(void); 
BYTE FAR PASCAL iBNext(void);
BYTE FAR PASCAL iBDataByte(BYTE);
BYTE FAR PASCAL iBSetup(BYTE);
BYTE FAR PASCAL iBAccess(void);
BYTE FAR PASCAL iBGndtest(void);
BYTE FAR * FAR PASCAL iBROMData(void);
BYTE FAR PASCAL iBKeyOpen(void);
BYTE FAR PASCAL iBKeyClose(void);
BYTE FAR PASCAL iBDOWCheck(void);
BYTE FAR PASCAL iBDataBit(BYTE);
BYTE FAR PASCAL iBOverdriveOn(void);
void FAR PASCAL iBOverdriveOff(void);
void FAR PASCAL FastSleep(WORD);
BYTE FAR PASCAL SetAdapterType(BYTE, char *);
BYTE FAR PASCAL SetAdapterSpeed(DWORD);
BYTE FAR PASCAL Adapter5VPrime(void);
BYTE FAR PASCAL Adapter5VCancel(void);
BYTE FAR PASCAL SetAdapter5VTime(BYTE);

#endif /* WIN16 */

/*
 * DLL Exported Function Prototypes
 */
BYTE DLLEXPORT SearchPorts(LPBYTE, LPBYTE, LPBOOL, LPIBHOST);
WORD DLLEXPORT SendCibMessage(BYTE, WORD, BYTE, PIBADDR);
WORD DLLEXPORT RecvCibMessage(BYTE, WORD, PIBADDR);
