/*****************************************************************************
 *                   
 *  Copyright (C) 1996, 97 Dallas Semiconductor Corporation. 
 *  All rights Reserved. Printed in U.S.A.
 *  This software is protected by copyright laws of
 *  the United States and of foreign countries.
 *  This material may also be protected by patent laws of the United States 
 *  and of foreign countries.
 *  This software is furnished under a license agreement and/or a
 *  nondisclosure agreement and may only be used or copied in accordance
 *  with the terms of those agreements.
 *  The mere transfer of this software does not imply any licenses
 *  of trade secrets, proprietary technology, copyrights, patents,
 *  trademarks, maskwork rights, or any other form of intellectual
 *  property whatsoever. Dallas Semiconductor retains all ownership rights.
 *
 *     Module Name: CIBCOMM
 *
 *     Description: 
 *
 *        Filename: cibcomm.c
 *
 * Dependant Files: cibcomm.h
 *
 *          Author: DL, TAC
 *
 *        Compiler: MSVC on Win32 x86, gcc on Solaris
 *
 *         Version: Ver 0.91 Beta
 *
 *         Created: 12/09/96
 *
 *   Modifications: TAC - 02/12/98  - Added iBStream calls.
 *                                  - Added "iButton for Java" specific code.
 *                      - 02/25/98  - Fixed Setup(), KeyOpen calling order
 *                                    problem.
 *                      - 03/02/98  - Added fix for Windows 98 comm problem.
 *
 *        Comments: - Crypto iButton specific code should be solid - not many
 *                    changes since late 1996 - though iBStream is new. iButton 
 *                    for Java specific code is Beta.
 *                  - Win32 buggy with speeds > 19200 -> Max speed set to 19200
 *
 *****************************************************************************/

/*
 * Uncomment next line to run to the rails !
 */
#define __NO_SLACK__

#include <memory.h>
#include <stdlib.h>

#ifdef WIN32
#include "sauth95.h"
#endif 

#include "cibcomm.h"

#ifdef WIN32
static int    InitialPriority;
static DWORD  InitialClass;
static HANDLE ProcessID, 
              ThreadID;
// Win98 vars
static OSVERSIONINFO VersionInfo  = {sizeof(OSVERSIONINFO),0,0,0};
static IsWin98                    = FALSE;
static HaveOsVersion              = FALSE;
//
#endif

#define STREAM_OVERHEAD 4

int   StreamLen = 0;
BYTE  StreamBuff[256+STREAM_OVERHEAD];
BYTE  LastSegment = FALSE;

void SetNewPriority()
{
#ifdef WIN32
  /* Get thread ID */
  ThreadID = GetCurrentThread();
  /* Get process ID */
  ProcessID = GetCurrentProcess();

  /* Get threads priority class */
  InitialClass = GetPriorityClass(ProcessID);
  /* Get current priority */
  InitialPriority = GetThreadPriority(ThreadID);

  /* Dial up the priority of this thread */
  SetPriorityClass(ProcessID, HIGH_PRIORITY_CLASS);
  SetThreadPriority(ThreadID, THREAD_PRIORITY_TIME_CRITICAL);
#endif
}

void RestoreOldPriority()
{
#ifdef WIN32
  /* Restore original thread priority level */
  SetPriorityClass(ProcessID, InitialClass);
  SetThreadPriority(ThreadID, InitialPriority);
#endif
}

BOOL DriverLoaded = FALSE;
BOOL PowerOn      = FALSE;
BOOL FirstCall    = TRUE;

BYTE DLLEXPORT SearchPorts(LPBYTE   lpROMBuffer, 
                           LPBYTE   lpROMPorts, 
                           LPBOOL   lpAdapterFound,
                           LPIBHOST lpiBHostPorts)
{
  BYTE NumFound = 0; 
  BYTE i; 

  if(FirstCall)
  {
    FirstCall = FALSE;
    GenCRC16Table(); 
  }

  *lpAdapterFound = FALSE;

#ifndef CIB_UNIX
#ifdef  WIN32 // kludge for Win98 Comm bug - 98 GPFs if speed > 19200
  if(!HaveOsVersion)
  {
    HaveOsVersion = GetVersionEx(&VersionInfo);

    if(VersionInfo.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS)
      IsWin98 = (VersionInfo.dwMajorVersion >= 4)&&
                (VersionInfo.dwMinorVersion >= 1);
  }
#endif

  if(!DriverLoaded)
    DriverLoaded = iBDOWCheck();
 
  if(DriverLoaded) 
#else
  if(TRUE)
#endif
  {
    for(i  = 0; i < lpiBHostPorts->AdapterCount; i++)
    {
      /* Tell the access system which type of adapter (i.e. parallel) */
      if(SetAdapterType(lpiBHostPorts->lpAdapter[i]->Type,
                          lpiBHostPorts->lpAdapter[i]->DevString))
      {
        iBSetup(lpiBHostPorts->lpAdapter[i]->Num);
        iBKeyOpen();
        /* iBOverdriveOn(); */

        /* Set baud rate for DS2480 */
        if(lpiBHostPorts->lpAdapter[i]->Type == SERIAL_DS2480)
          SetAdapterSpeed(9600);

        /* Find first 1-wire device on selected adapter */
        if(iBFirst())
        {
          *lpAdapterFound = TRUE;
  
           do
           {
             /* Test family code */ 
             if(*iBROMData() == CIB_FC)
             {
                /* Copy ROM data to caller's buffer */
                memcpy(lpROMBuffer + (NumFound << 3), iBROMData(), 8);
                /* Assign port number */
                lpROMPorts[NumFound++] = i;
             }
           }while (iBNext() && (NumFound < MAX_CIB_NUM));
        }

        iBKeyClose();
      }
    }
  }

  return NumFound;
}
/****************************************************************************/

#ifdef __NO_SLACK__
static WORD MinRun = 64;
#else
static WORD MinRun = 96;
#endif

/****************************************************************************/
void SetMinRunTime(WORD MR)
{
   MinRun = MR;
}
/****************************************************************************
 *
 * Top-Level function to send a message to the CiB
 *                                                                     
 *      mp        = Pointer to message string
 *      mlen      = Length of message (bytes)
 *      ExecTime  = Estimated execution time (0-15, where 0 = minimum)
 *      lpBuf     = Pointer to "garbage bag" storage
 *
 *      return    = Error code, or zero if all went well
 *
 */
WORD DLLEXPORT SendCiBMessage(BYTE *mp, 
                              WORD mlen, 
                              BYTE ExecTime, 
                              PIBADDR lpBuf)
{
  BYTE ETime;         /* Temp value for ExecTime */
  BYTE Retry;         /* Retry counter */
  BYTE seg;           /* Current segment number */
  BYTE InFree;        /* Inbound FIFO bytes available */
  BYTE OutUsed;       /* Outbound FIFO bytes used */
  BYTE owms;          /* micro status */
  BYTE cpst;          /* Co-Processor status */
  WORD checksum = 0;  /* checksum accumulator */
  WORD sum = 0;       /* Temporary checksum store */
  BYTE error;        

  seg = 127;          /* Start with no segment number done */

  ExecTime &= 0xFF;
  ETime = (BYTE) ExecTime;

#ifndef CIB_UNIX
  if(!DriverLoaded)
    DriverLoaded = iBDOWCheck();
#endif

  SetNewPriority();

  if(!SetAdapterType(lpBuf->iBPort.Type, lpBuf->iBPort.DevString))
    return ER_ADAPTER;
  
  iBSetup(lpBuf->iBPort.Num);
  iBKeyOpen();
  iBOverdriveOn();

  /* Set baud rate for DS2480 */
  if(lpBuf->iBPort.Type == SERIAL_DS2480)
  {
#ifndef CIB_SOLARIS_SPARC /* SPARC serial port can't do higher speeds */
/************** GPFs in Windows 98 if speed > 19200 **************************
 * Okay in OSR2, NT 4.0 and NT 5.0
 * Windows 98 comm port problems have been seen before!!!
 */
#ifdef WIN32
      if(IsWin98)
        SetAdapterSpeed(19200);
      else
#endif// WIN32
      if(!SetAdapterSpeed(57600))
#endif// CIB_SOLARIS_SPARC
        SetAdapterSpeed(19200);
  }

  /* Copy ROM data to access system's ROM data buffer */
  memcpy(iBROMData(), lpBuf->ROMData, 8);

  /* Set-up a Retry loop */
  for(Retry = 0; Retry < 3; Retry++) 
  {
    while(1)
    {      
      /* Get the status from the device - */
      if((error = GetStatus(&InFree, &OutUsed, &owms, &cpst)) != 0)
        break;

      /* See if a POR has occurred and treat it as an error if so - */
      if(owms & CE_BPOR)
      {
        error = ER_POR;
        break;
      }

      /* See if the co-processor was running when we stopped - */
      if(cpst & 1)
      {
        /* Execute the co-processor and allow it to finish up - */
        if((error = SendRun()) != 0)
          break;

        TimeDelay((BYTE) (ETime + 6));
      }
      else
      {
        /* If the co-processor is not running, then - */

        /* Work around for 0 owms */
        if(!(owms & 0x7f) && !OutUsed)
          owms = 0x3F;

        /* See if the previous command has not yet been completed - */
        if((owms & 0x20) || 
           ((owms & 0x1F) == CE_FIRSTBIRTH) ||
           (owms == 0x1F))
        {
          /* 
           * Decide on a time value to complete the prior operation - 
           * For now, we'll send back 1/2 of the status value as a time - 
           */
          ETime = ((owms & 0x1F) >> 1);

          if(ETime == 0)
            ETime++;

          /* Send the time value to the CIB - */
          if((error = SendStatus(ETime)) != 0)
            break;

          /* Execute the micro and allow it to finish up - */
          if((error = SendRun()) != 0)
            break;

          /*
           *    Wait an extra 1.5 seconds to compensate for alarm cycling 
           * problem.
           */
          TimeDelay(6);
        }
        else
        {
          /* See if the device has room for new command. Error if not - */
          if(InFree < HEADERSIZE)
          {
            error = ER_NOROOM;
            break;
          }

          /*
           * If the segment we're about to send is not the FIRST one,
           * check the status to see if we had any comm errors on the last
           * (Errors left over from prior uses don't concern us!)
           */
          if(seg != 127)
          {
            if(((owms & 31) > 1) & ((owms & 31) < 8))
            {
              error = (owms & 31) | ER_COMMERROR;
              break;
            }
          }
          
          /* If the segment number MS bit = 1, then we're done */

          if(seg & 128)
          {
            RestoreOldPriority();

            /* Successful exit */
            return 0;         
          }

          /* When we have finished a segment, go to the next segment */
          seg = (++seg) & 127;

          /*If about to send iBFirst segment, clear checksum accumulator*/
          checksum = seg ? checksum + sum : 0;

          /*
           * For each new segment that we attempt to send out, we start a
           * temporary checksum in 'sum' with the current checksum value. 
           * If the segment transfer fails, we will not have added 
           * anything to the actual checksum so we can try again
           */
          sum = checksum;
          
          /* Send out the next segment */
          if((error = SendSegment(&seg, mp, mlen, &sum)) != 0)
            break;
             
          /*
           * Now we have loaded the data segment into IPR register and
           * the header for it into the FIFO. Next set the execution time
           * if it's the last message segment, and the execute the message
           */
          if(seg & 128)
          {
            ETime = (BYTE) ExecTime;

            if((error = SendStatus(ETime)) != 0)
                break;
          }
          else
            ETime = 0;

          /* Send the run-interrupt command to the device */
          if((error = SendInterrupt()) != 0)
             break;
        }

#ifndef __NO_SLACK__
        TimeDelay((BYTE) (ETime + 1));
#else
        TimeDelay(ETime);
#endif
      }

    } /* End of forever loop */

    /* Save the error cause - */
    ETime = error;

    /* If we had an error and broke out of the loop, reset the micro */
    if((error = SendReset()) != 0)
      break;

    /* Then allow it to run and report a completion status */
    if((error = SendRun()) != 0)
      break;

    /* See if a little run time will cure the problem that caused the reset*/
    TimeDelay(4);

    /* We always begin again at the start (segment 0) after an error */
    seg = 127;
    
    /* Restore the original error cause code - */
    error = ETime;

  } /*End of Retry loop */

  iBOverdriveOff();
  iBKeyClose();
  RestoreOldPriority();

  /* Error-return if we failed all the retries */
  return error;
}
/****************************************************************************
 *
 * Send out one (specific) segment of the message to the device
 *
 *      pseg   = Pointer to current segment number (0 = iBFirst)
 *      mp     = Pointer to message string
 *      msglen = Length of message string (bytes)
 *      psum   = Pointer to checksum accumulator
 *
 *      return =   Error code, or zero if all went well
 *
 */
BYTE SendSegment(LPBYTE pseg, LPBYTE mp, WORD msglen, LPWORD psum)
{
  BYTE   seglen;      /* Length of segment we're sending */
  WORD   index;       /* Offset of segment we're sending */
  BYTE   remain;      /* Bytes remaining before this seg */
  LPBYTE segstart;    /* Start of segment in message */
  BYTE   error;

  /* Offset of next segment to send */
  index = (*pseg * SEGSIZE);     

  /* Computer how much data remains unsent to the device */
  remain = msglen - index;      /* Length of segment remaining send */

  /* Limit the segment size to that which is allowed in this device */
  if(remain > SEGSIZE)
  {
    /* If remain is bigger than max, send max */
    seglen = SEGSIZE;          
  }
  else
  {
    /* If remain will fit, wrap it up */
    seglen = remain;           
    /* Flag this as the last segment */
    *pseg |= 128;  
  }

  /* Define the point in the message where this segment begins */
  segstart = mp + index;

  /* Send out the segment data to the device */
  if((error = SendData(segstart, seglen, psum)) != 0)
    return error;

  /* Send out the header to the device */
  if((error = SendHeader(*pseg, segstart, remain, seglen, psum)) != 0)
    return error;

  return 0;
}
/****************************************************************************
 *
 * Send out the data portion of a message segment to the device
 *
 *      mp        =    Pointer to segment in the message string
 *      seglen    =    length of this segment
 *      psum      =    Pointer to checksum accumulator
 *
 *      return    =   Error code, or zero if all went well
 *
 */
BYTE SendData(LPBYTE mp, BYTE seglen, LPWORD psum)
{
  BYTE  Retry, 
        error;
  BYTE  i;  
  WORD  crc;
 
  /* Now try (and Retry) to output the data without error */
  for (Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      /* Send out the data segment to the device and compute the crc */
      /* and checksum as it goes out */

      StreamLen = 0;
      
      StreamBuff[StreamLen++] = OW_IPRWRITE;
      StreamBuff[StreamLen++] = seglen;
      
      for(i = 0; i < seglen; i++)
        StreamBuff[StreamLen++] = mp[i];
        
      /* set up the crc bytes */
      StreamBuff[StreamLen++] = 0xff;           
      StreamBuff[StreamLen++] = 0xff;
      
      if(iBStream(StreamBuff,StreamLen) == 0)
      {
        error = ER_ACCESS;
        break;
      }

      for(i = 0; i < seglen; i++)
        mp[i] = StreamBuff[i+2];
      
      crc = CRC16Buf(StreamBuff, 0, (WORD)(StreamLen-2));

      for(i = 0; i < seglen; i++)
        *psum += mp[i];
     
      /* Check the returning CRC from the device */
      if((error = CheckStreamCRC(StreamBuff,StreamLen-2,crc)) != 0)
        break;

      /* If all has gone well, return with AOK status */
      return 0;
    }
  }

  /* If we've retried and failed, return with failure status */
  return (error | ER_SENDDATA);
}
/****************************************************************************
 *
 * Send out the header portion of a message segment to the device
 *
 *      seg    =   Segment number to send out (0-127)
 *      mp     =   Pointer to segment in message string
 *      remain =   Number of bytes remaining, including this segment
 *      psum   =   Pointer to checksum accumulator
 *
 *      return =   Error code, or zero if all went well
 *
 */
BYTE SendHeader(BYTE seg, LPBYTE mp, WORD remain, BYTE seglen, LPWORD psum) 
{
  WORD      crc, 
            segcrc;
  BYTE      Retry, 
            error;
  FIFORDWR  FB;
  WORD      i;

  /*
   * We must compute the data segment's CRC for the header
   * This includes the segment length value
   */
  segcrc = CRC16(seglen, 0);
  segcrc = CRC16Buf(mp, segcrc, seglen);

  /* Now try (and Retry) sending the header to the device */
  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      if(!iBFastAccess())
      {
        error = ER_ACCESS;
        break;
      }

      FB.Cmd = OW_FIFOWRITE;
      FB.Len = FIFO_LEN;

      FB.FIFOBuf[0] = seg;
      FB.FIFOBuf[1] = seglen;
      FB.FIFOBuf[2] = LSB(remain);
      FB.FIFOBuf[3] = MSB(remain);
      FB.FIFOBuf[4] = LSB(segcrc);
      FB.FIFOBuf[5] = MSB(segcrc);

      for(i = 0; i < 6; i++)
        *psum += FB.FIFOBuf[i];

      FB.FIFOBuf[6] = LSB(*psum);
      FB.FIFOBuf[7] = MSB(*psum);

      iBDataBlock(&FB.Cmd, 10);
      crc = CRC16Buf(&FB.Cmd, 0, 10);
		 
      /*
       * Add the checksum bytes to the checksum for subsequent segments
       * (Uses segcrc because it's handy!)
       */
      segcrc = *psum;
      *psum += LSB(segcrc);
      *psum += MSB(segcrc);

      /* Check the returning CRC from the device */
      if((error = CheckCRC(crc)) != 0)
        break;

      /* Send the release sequence and check the ACK bit */
      if((error = ReleaseSequence(RS_FIFOWRITE)) != 0)
        break;

      /* If all went well, return with OK status */
      return 0;
    }
  }

  /* If we retried and failed, return with failure status */
  return (error | ER_SENDHDR);
}
/****************************************************************************
 *
 * Return the device status values -
 *
 *    InFree    =   Pointer to status = Number of free bytes in FIFO
 *    OutUsed   =   Pointer to status = Number of bytes waiting in FIFO
 *    owms      =   Pointer to status = One-Wire Micro Status byte
 *    cpst      =   Pointer to status = Co-Processor Status Byte
 *
 *    return    =   Error code, or zero if all went well
 *
 */
BYTE GetStatus(LPBYTE InFree, LPBYTE OutUsed, LPBYTE owms, LPBYTE cpst)
{
  BYTE  IOBuf[5];
  BYTE  Retry, 
        error;
  WORD  crc;
  int   i;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      StreamLen = 0;
      
      StreamBuff[StreamLen++] = OW_STATUSREAD;

      for(i = 0; i < 6; i++) /* 2 for CRC read */
        StreamBuff[StreamLen++] = 0xff;
        
      if(iBStream(StreamBuff,StreamLen)==0)
      {
        error = ER_ACCESS;
        break;
      }
      
      for(i = 0; i < 5; i++)
        IOBuf[i] = StreamBuff[i];

      crc = CRC16Buf(StreamBuff, 0, (WORD)(StreamLen-2));
      
      *InFree  = IOBuf[1];
      *OutUsed = IOBuf[2];
      *owms    = IOBuf[3];
      *cpst    = IOBuf[4];
      
      if((error = CheckStreamCRC(StreamBuff,StreamLen-2,crc)) != 0)
        break;

      /* If success, return an AOK status */
      return 0;
    }
  }

  /* If we've re-tried and failed, return a failure status */
  return (error | ER_GETSTATUS);
}
/****************************************************************************
 *
 *   Send the owus byte (timing value) to the device -
 *
 *      owus        =   One-Wire Uart Status byte
 *
 *      return      =   Error code, or zero if all went well
 *
 */
BYTE SendStatus(BYTE owus)
{
  BYTE  Retry, 
        error;
  WORD  crc;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      if(!iBFastAccess())
      {
        error = ER_ACCESS;
        break;
      }

      crc = 0;
      crc = CRC16(iBDataByte(OW_STATUSWRITE), crc);
      crc = CRC16(iBDataByte(owus), crc);

      if((error = CheckCRC(crc)) != 0)
        break;

      if((error = ReleaseSequence(RS_STATUSWRITE)) != 0)
        break;

      /* If all is well, exit with OK status */
      return 0;
    }
  }

  /* If retries failed, return with failure status */
  return (error | ER_SENDSTATUS);
}

/****************************************************************************
 *
 * The SendRun function sends the micro-run command to the device
 *
 *      return      =   Error code, or zero if all went well
 *
 */
BYTE SendRun()
{
  BYTE  Retry, 
        error;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      if(!iBFastAccess())
      {
        error = ER_ACCESS;
        break;
      }

      iBDataByte(OW_MICRORUN);

      /* Issue the release sequence code */
      if((error = ReleaseSequence(RS_MICRORUN)) != 0)
        break;

      /* If all went well, return with AOK status */
      return 0;
    }
  }

  /* If errors occurred return error status */
  return (error | ER_SENDRUN);
}
/****************************************************************************
 *
 * The SendReset function sends the micro-reset command to the device
 *
 *      return = Error code, or zero if all went well
 *
 */
BYTE SendReset()
{
  BYTE Retry, 
       error;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      if(!iBFastAccess())
      {
        error = ER_ACCESS;
        break;
      }

      iBDataByte(OW_MICRORESET);

      if((error = ReleaseSequence(RS_MICRORESET)) != 0)
        break;

      /* If all went well, return w/o error */
      return 0;
    }
  }

  return (error | ER_SENDRESET);
}
/****************************************************************************
 *
 * The SendInterrupt function sends the micro-interrupt command to the device
 *
 *      return = Error code, or zero if all went well
 *
 */
BYTE SendInterrupt()
{
  BYTE Retry, 
       error;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      if(!iBFastAccess())
      {
        error = ER_ACCESS;
        break;
      }

      iBDataByte(OW_MICROINTERRUPT);

      if((error = ReleaseSequence(RS_MICROINTERRUPT)) != 0)
        break;

      return 0;
    }
  }

  return error | ER_SENDINT;
}


/****************************************************************************
 *
 * Return Message Function -
 *
 *    mp      =   Current Pointer to message buffer string
 *    bufsize =   Number of bytes in provided buffer space
 *
 *    return  =   Error code, or zero if all went well
 *
 */
WORD DLLEXPORT RecvCiBMessage(BYTE *mp, WORD *bufsize, PIBADDR lpBuf)
{
  BYTE ETime;          /* Temporary time value */
  BYTE *mbuf;          /* Start of message buffer */
  BYTE InFree;         /* Inbound FIFO bytes available */
  BYTE OutUsed;        /* Outbound FIFO bytes used */
  BYTE owms;           /* micro status */
  BYTE cpst;           /* Co-Processor status */
  BYTE seg;            /* Current segment number */
  WORD hdrcrc;         /* Crc from header */
  WORD hdrsum;         /* Checksum from header */
  WORD checksum;       /* Running checksum of the message */
  WORD sum;            /* Temporary checksum store */
  BYTE size;           /* Size of current segment */
  WORD msglen;         /* Length of message received so far */
  BYTE oldseg;         /* Prior segment number for checking order */
  BYTE Retry, 
       error;
  BYTE JavaiButton = 0;        

  checksum = msglen = 0;
  /* Remember where the buffer starts */
  mbuf = mp;              

  SetNewPriority();

  /* Copy ROM data to access system's ROM data buffer */
  memcpy(iBROMData(), lpBuf->ROMData, 8);

  /* Set up a loop to try (and Retry) getting a message from the device */
  for(Retry = 0; Retry < 6; Retry++)
  {
    /* When we Retry, Retry from the top - */
    oldseg = 0xFF;

    while(1)
    {
      /*Get the device status and see if there is a segment there to read*/
      if((error = GetStatus(&InFree, &OutUsed, &owms, &cpst)) != 0)
        break;

      /* See if a POR has occurred and treat it as an error if so - */
      if(owms & CE_BPOR)
      {
        error = ER_POR;
        break;
      }

      /* See if the co-processor was running when we stopped - */
      if((cpst & 1) != 0)
      {
        /* Execute the co-processor and allow it to finish up - */
        if((error = SendRun()) != 0)
          break;

        /* Compute a delay time to complete - */
        ETime = cpst >> 4;

        /* Wait for whatever we did above to finish up - */
        TimeDelay(ETime);
      }
      else
      {
        /* If the co-processor is not running, then - */

        /* See if the previous command has not yet been completed - */
        if(owms & 0x20) 
        {
          /*
           * Decide on a time value to complete the prior operation -
           * For now, we'll send back 1/2 of the status value as a time -
           */
          ETime = ((owms & 0x1F) >> 1);

          if(ETime == 0)
            ETime++;

          /* Send the time value to the CIB - */
          if((error = SendStatus(ETime)) != 0)
            break;

          /* Execute the micro and allow it to finish up - */
          if((error = SendRun()) != 0)
            break;

          /* Wait for whatever we did above to finish up - */
          TimeDelay(ETime);
        }
      }

      /* See if there's a segment header waiting for us - */
      if(OutUsed == 0)
      {
        error = ER_NOHEADER;
        break;
      }

      /* See if the header is valid (ie: the correct length) */
      if(OutUsed != HEADERSIZE)
      {
        error = ER_BADHEADER;
        break;
      }

      /* Read the header in from the device */
      if((error = RecvHeader(&seg, &hdrcrc, &hdrsum, &size, &sum)) != 0)
        break;

      /* See if this is the first segment of a new message */
      if(((seg & 127) == 0) && (!JavaiButton))
      {
        /*
         * When we get the first segment of a new message we must
         * initialize everything
         */
        msglen = checksum = 0;
        oldseg = 0;
        mp     = mbuf;
      }
      else
      {
        if(JavaiButton)
        {
          checksum = 0;
          seg = oldseg = 0;
        }
        /* See if the segments are in proper order */
        else if (++oldseg != (seg & 127))
        {
          error = CE_BADSEQ;
          break;
        }
      }

      /* See if the new message data would overrun the buffer */
      if((msglen + size) > *bufsize)
      {
        error = ER_OVERRUN;
        break;
      }

      /* Make sum equal the whole progressive checksum including header */
      checksum += sum;

      /* Read in the segment data */
      if((error = RecvData(mp, size, hdrcrc, &sum, JavaiButton)) != 0)
        break;

      /* If the message data came in OK then update values */
      checksum += sum;        /* Keep the end-to-end checksum current */
      
      if(!JavaiButton)
      {
        msglen   += size;       /* Keep track of the size of the message */
        mp       += size;       /* Advance the message pointer */
      }
      else
      {
        msglen   += size - 3;       /* Keep track of the size of the message */
        mp       += size - 3;       /* Advance the message pointer */
      }
      
      /* Verify the progressive checksum value - */
      if(hdrsum != checksum)
      {
        error = ER_CHECKSUM;
        break;
      }

      /* Keep the running sum correct */
      checksum += LSB(hdrsum);
      checksum += MSB(hdrsum);

      /*
       * See if this was the last message segment. If it was, then we
       * return with AOK status, and rcvd message size replaces bufsize.
       */
      if(LastSegment && (owms != 14))
      {
        iBOverdriveOff();
        iBKeyClose();

        *bufsize = msglen;

        RestoreOldPriority();
        return 0;
      }
      else if(owms == 14)
        JavaiButton = 1;

      /* Allow the micro to run and bring up our next segment - */
      if(SendRun() != 0)
        break;

      TimeDelay(0);
    }    

    /* If we have any failure, we must alow the micro to run or we */
    /* may not recover from it - */
    if(SendRun() == 0)
      TimeDelay(0);
  }

  iBOverdriveOff();
  iBKeyClose();
  RestoreOldPriority();

  /* Return with error status if retries exhausted */
  return error;
}
/****************************************************************************
 *
 * Read a header (FIFO) from the device -
 *
 *    seg    = Pointer to segment number from the device
 *    hdrcrc = CRC-16 of the segment data area from the header
 *    hdrsum = Running checksum result from the header
 *    size   = Segment data area size from the header
 *    sum    = Checksum of the header bytes
 *
 *    return = Error code, or zero if all went well
 *
 */
BYTE RecvHeader(LPBYTE seg, 
                LPWORD hdrcrc, 
                LPWORD hdrsum, 
                LPBYTE size, 
                LPWORD sum)
{
  WORD     msglen;
  WORD     owcrc;
  WORD     i;
  FIFORDWR FB;
  BYTE     Retry, 
           error;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      FB.Cmd = OW_FIFOREAD;
      FB.Len = FIFO_LEN;
      memset(FB.FIFOBuf, 0xFF, 8);

      StreamLen = 0;
      
      memcpy(StreamBuff,&FB.Cmd,10);
      StreamLen += 10;
      
      /* set up crc */
      StreamBuff[StreamLen++] = 0xff;
      StreamBuff[StreamLen++] = 0xff;
      
      if(iBStream(StreamBuff,StreamLen)==0)
      {
        error = ER_ACCESS;
        break;
      }

      memcpy(&FB.Cmd,StreamBuff,10);
      owcrc = CRC16Buf(&FB.Cmd, 0, 10);

      *sum = 0;
      
      for(i = 0; i < 6; i++)
        *sum += FB.FIFOBuf[i];

      *seg    = FB.FIFOBuf[0];
      *size   = FB.FIFOBuf[1];
      msglen  = (FB.FIFOBuf[3] << 8) | FB.FIFOBuf[2];
      *hdrcrc = (FB.FIFOBuf[5] << 8) | FB.FIFOBuf[4];
      *hdrsum = (FB.FIFOBuf[7] << 8) | FB.FIFOBuf[6];

      if(((*seg) & 128) == 128)
        LastSegment = TRUE;
      else
        LastSegment = FALSE;  

      if((error = CheckStreamCRC(StreamBuff,StreamLen-2,owcrc)) != 0)
        break;

      if((error = ReleaseSequence(RS_FIFOREAD)) != 0)
        break;

      /* If all went well, return OK status */
      return 0;
    }
  }

  /* If retries exhautsed, return error status */
  return (error | ER_RECVHDR);
}
/****************************************************************************
 *
 * Return the segment data from the device
 *
 *    mp     = Pointer to message buffer next available position
 *    size   = Size of message segment (from header)
 *    hdrcrc = CRC-16 of the data segment from header
 *    sum    = Checksum accumulator for segment data
 *
 *    return = Error code, or zero if all went well
 *
 */
BYTE RecvData(LPBYTE mp, BYTE size, WORD hdrcrc, LPWORD sum, BYTE JavaiButton)
{
  LPBYTE mstart; 
  WORD   crc, 
         owcrc, 
         cksum, 
         n;     
  int    i;
  BYTE   Retry, 
         error;
  LPBYTE TempArray;                 

  mstart = mp;

  for(Retry = 0; Retry < 3; Retry++)
  {
    while(1)
    {
      /* Get the current buffer pointer */
      mp = mstart;

      crc = CRC16(size,0);

      /* Initialize the local checksum accumulator */
      cksum = 0;
      
      if(!JavaiButton)
      {      
        StreamLen = 0;
      
        StreamBuff[StreamLen++] = OW_IPRREAD;
        StreamBuff[StreamLen++] = size;
      
        for(i = 0; i < size+2; i++)/* 2 for crc read */
          StreamBuff[StreamLen++] = 0xff;
      
        if(iBStream(StreamBuff,StreamLen)==0)
        {
          error = ER_ACCESS;
          break;
        }        
      
        for(i = 0; i < size; i++)
          mp[i] = StreamBuff[i+2];

        owcrc = CRC16Buf(StreamBuff,0,(WORD)(size+2));
        crc   = CRC16Buf(StreamBuff+1,0,(WORD)(size+1));

        for(n = 0; n < size; n++)
          cksum += mp[n];
     
        /* Check the returning CRC values against what we received */
        if((error = CheckStreamCRC(StreamBuff,StreamLen-2,owcrc)) != 0)
          break; 

        /* See if the CRC of the data matches what we expected as-per header */
        if(hdrcrc != crc)
        {
          error = ER_HDRCRC;
          break;
        }

        /* Since we've been successful, update the checksum and return AOK */
        *sum = cksum;
        
        return 0;
      } /* if(!JavaiButton) */
      else
      {
        if(!(TempArray = malloc(size*2)))
        {
          error = ER_OVERRUN;
          break;
        }
        
        StreamLen = 0;
      
        StreamBuff[StreamLen++] = OW_IPRREAD;
        StreamBuff[StreamLen++] = size;
      
        for(i = 0; i < size+2; i++)/* 2 for crc read */
          StreamBuff[StreamLen++] = 0xff;
      
        if(iBStream(StreamBuff,StreamLen)==0)
        {
          error = ER_ACCESS;
          free(TempArray);
          break;
        }        
      
        for(i = 0; i < size; i++)
          TempArray[i] = StreamBuff[i+2];

        owcrc = CRC16Buf(StreamBuff,0,(WORD)(size+2));
        crc   = CRC16Buf(StreamBuff+1,0,(WORD)(size+1));

        for(n = 0; n < size; n++)
          cksum += TempArray[n];
     
        /* Check the returning CRC values against what we received */
        if((error = CheckStreamCRC(StreamBuff,StreamLen-2,owcrc)) != 0)
          break; 
                   
        /* See if the CRC of the data matches what we expected as-per header */
        if(hdrcrc != crc)
        {
          error = ER_HDRCRC;
          free(TempArray);
          break;
        } 

        /* Since we've been successful, update the checksum and return AOK */
        *sum = cksum; 
        memcpy(mp, TempArray+3,size-3);                               
        free(TempArray);
        return 0;
      }/* else */
    } /* for */
  }/* for Retry */

  /* If retries failed, return the error status */
  return (error | ER_RECVDATA);
}
/****************************************************************************
 *
 * Issue the release sequence to the device and check the ACK response
 *
 *    rscode = Release sequence code (command specific)
 *
 *    return = Error code, or zero if all went well
 *
 */
BYTE ReleaseSequence(WORD rscode)
{
  BYTE IOBuf[20];

  IOBuf[0] = LSB(rscode);
  IOBuf[1] = MSB(rscode);

  iBDataBlock(IOBuf, 2);

  if((rscode == RS_MICRORUN) || (rscode == RS_MICROINTERRUPT))
  {
    PowerOn = TRUE;
    SetAdapter5VTime(PARMSET_infinite);
    Adapter5VPrime();
  }

  /* ACK bit from device (should be zero) - */
  return !iBDataBit(TRUE) ? 0 : ER_ACK;
}

/****************************************************************************
 *
 * Check the returning crc bytes against the crc provided (expected) -
 *
 *      crc    =   CRC16 result of received bytes from device
 *
 *      return =   Error code, or zero if all went well
 *
 */
BYTE CheckCRC(WORD crc)
{
  BYTE IOBuf[2];

  IOBuf[0] = IOBuf[1] = 0xFF; 

  iBDataBlock(IOBuf, 2);

  /* Verify the CRC LS byte response */
  if(IOBuf[0] != (BYTE) ~LSB(crc))
    return ER_CRC;

  /* Verify the CRC MS byte response */
  if(IOBuf[1] != (BYTE) ~MSB(crc))
    return ER_CRC;

  return 0;
}
/****************************************************************************
 *
 * Check the returning crc bytes in stream buffer bp against the crc provided 
 * (expected) -
 *
 *      data   =   pointer to stream buffer
 *      offset =   offset to first SRS byte
 *      crc    =   CRC16 result of received bytes from device
 *
 *      return =   Error code, or zero if all went well
 *
 */
BYTE CheckStreamCRC(LPBYTE data, int offset,  WORD crc)
{

  if(data[offset] != (LSB(crc)^0xFF))
  {
    return ER_CRC;
  }

  if(data[offset+1] != (MSB(crc)^0xFF))
  {
    return ER_CRC;
  }

  return 0;
}
/****************************************************************************
 *
 * Produce a time delay as-per et -
 *
 *      return      =   None
 *
 */
void TimeDelay(BYTE et)
{
  FastSleep((WORD)(et*250 + MinRun));

  if(PowerOn)
  {
    PowerOn = FALSE;
    Adapter5VCancel();
  }
}
/****************************************************************************
 *
 *   CRC16 generation. Used here only for table generation.
 *
 */
WORD CRC16TGen(BYTE x, WORD crc)
{
  BOOL bit;
  BYTE n;

  for(n = 0; n < 8; n++)
  {
    bit = (crc ^ x) & 1;
    crc >>= 1;

    if(bit)
      crc ^= 0xA001;

    x >>= 1;
  }

  return crc;
}

WORD CRCTable[256];

/****************************************************************************
 *
 *    Generate 256 word table for CRC16 lookup.
 *
 */
void GenCRC16Table()
{
  WORD i;

  for(i = 0; i < 256; ++i)
    CRCTable[i] = CRC16TGen((BYTE) i, 0);
}
/****************************************************************************
 *
 *    Fast CRC16 generation.
 *
 */
WORD CRC16(BYTE x, WORD crc)
{
  return CRCTable[(crc ^ x) & 0xFF] ^ crc >> 8;
}
/****************************************************************************
 *
 *    Compute CRC16 of buffer pointed to by bp.
 *
 */
WORD CRC16Buf(LPBYTE bp, WORD crc, WORD Count)
{
  WORD i;

  for(i = 0; i < Count; i++)
    crc = CRCTable[(crc ^ bp[i]) & 0xFF] ^ crc >> 8;

  return crc;
}
