/*****************************************************************************
 *
 *    Access system layer the the CiB API.          { Ver 0.01  10/24/96 }
 *         
 *   Copyright (C) 1994, 1996 Dallas Semiconductor Corporation. 
 *   All rights Reserved. Printed in U.S.A.
 *   This software is protected by copyright laws of
 *   the United States and of foreign countries.
 *   This material may also be protected by patent laws of the United States 
 *   and of foreign countries.
 *   This software is furnished under a license agreement and/or a
 *   nondisclosure agreement and may only be used or copied in accordance
 *   with the terms of those agreements.
 *   The mere transfer of this software does not imply any licenses
 *   of trade secrets, proprietary technology, copyrights, patents,
 *   trademarks, maskwork rights, or any other form of intellectual
 *   property whatsoever. Dallas Semiconductor retains all ownership rights.
 *
 *****************************************************************************
 *
 *  SAUTHUNX.C
 *  Software Authorization Comm Layer.           
 *
 *
 *  Author: 
 *  Project: 
 *  Created: ?
 *  Distribution: In House
 *  Compiler: GCC 2.6.3 (tested under Linux)
 *  Externals: ds2480com.c ds2480com.h sauthunx.h serial.h
 *  Version: 1.00.0
 *
 *  Version History:
 *  1.00.0 - Added Version header.  iBStrongAccess() was not detecting loss
 *           of communication with button.
 *           2/11/98
 */

#include <fcntl.h>  
#include <signal.h> 
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/stat.h>
#include <errno.h>

#include "ds2480.h"
#include "ds2480com.h"
#include "serial.h"
#include "sauthunx.h"

/* 
 * State and mode flags 
 */
uchar SetupOk = FALSE; 
uchar FailNext;
uchar AccessGood;
uchar Passthru = TRUE;
uchar AuthFlag = FALSE;

/* Semaphore stuff */
static uchar sem_init = 0;
static int   sem_id;
static struct sembuf sops;
/* End of Semaphore stuff */

/* 
 * ROM search depth specifier 
 */
uchar l0 = 0;

/* 
 * ROM data buffer 
 */
uchar RomDta[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/* 
 * Xmit structure used for communication with parallel port driver 
 */
DOWComm DS1481Comm;

/* 
 * Port/Adapter configuration vars 
 */
int   FileDes;
uchar Type = SERIAL_DS2480;
uchar PortNum = 0;
char  FileName[MAXFILENAMELEN]; 
int   returnval;

uchar iBKeyOpen()
{
   int semflag;
   key_t DS1410_key;

#ifdef HAS_UNION_SEMUN
   union semun semarg;
#else
   union semun {
     int val;
     struct semid_ds *buf;
     ushort *array;
   } semarg;
#endif

   if (!sem_init)
   {
     /* Try to create semaphore */
     DS1410_key = DS1410_KEY;
     semflag = IPC_CREAT | IPC_EXCL | OUR_SEM_PERMS;
     sem_id = semget(DS1410_key, 1, semflag);
                  
     if (sem_id == -1)
     {
       /* Already created */
       semflag = IPC_EXCL | OUR_SEM_PERMS;
       sem_id = semget(DS1410_key, 1, semflag);
     }
     else
     {
       /* Initialize semval */
       semarg.val = 1;
       semctl(sem_id, 0, SETVAL, semarg);
     }

     sem_init = TRUE;
   }

   if (!AuthFlag)
   {
      /* Set up struct sops to change sem value, with UNDO */
      sops.sem_num = 0;
      sops.sem_op  = -1;
      sops.sem_flg = SEM_UNDO;

      /* Block here until semaphore is released. */
      semop(sem_id, &sops, 1);
                        
      /* Open the file */
      switch (Type)
      {
        case SERIAL_DS2480:
          FileDes = ser_gethandle(FileName);
          break;
        case PARALLEL_DS1481:
          /* If we've used iBSetup, PortNum will not be zero. 
             iBSetup denotes use of old driver call to /dev/dow0.
          */
          if (PortNum == 0)
            FileDes = open(FileName,O_RDWR | O_NONBLOCK);
          else
            FileDes = open("/dev/dow0",O_RDWR | O_NONBLOCK);
          break;
        default:
          FileDes = -1;
          break;
      }

      if (FileDes < 0)
      {
        /* Release and set semval back to 1 */
        semarg.val = 1;
        semctl(sem_id, 0, SETVAL, semarg);
        return(AuthFlag);
      }

      AuthFlag = TRUE;

      /* Get the adapter ready to work */
      switch (Type)
      {
        case SERIAL_DS2480:
          /* We need to set the speed to 9600 and send a reset first
             just in case the part is in calibration mode. 6/2/97 */
          ser_setspeed(FileDes,9600);
          /* This reset is necessary for the part to calibrate.
             New as of 6/2/97 */
/*          DS2480Reset(FileDes);*/
          DS2480ResetSyncNonBlock(FileDes);
          SetAdapterSpeed(9600);
          break;
        case PARALLEL_DS1481:
          ExitPassthru();
          break;
      }
   } 

   return AuthFlag;
}

uchar iBKeyClose()
{
#ifdef HAS_UNION_SEMUN
   union semun semarg;
#else
   union semun {
     int val;
     struct semid_ds *buf;
     ushort *array;
   } semarg;
#endif

   if (AuthFlag)
   {
      switch(Type)
      {
        case SERIAL_DS2480:
          ToggleCommandMode(FileDes);
          DOWReset();
          SetAdapterSpeed(9600);
          break;

        case PARALLEL_DS1481:
          EnterPassthru();
          break;
      }

      close(FileDes);
      
      /* Set semval back to 1 */
      semarg.val = 1;
      semctl(sem_id, 0, SETVAL, semarg);
      
#ifdef USES_NAP
      nap(32);
#else
      usleep(32);
#endif
      AuthFlag = FALSE;
   } 

   return TRUE;
}

/* 
 *   See if device driver has been installed 
 */
uchar iBDOWCheck()                 
{
  struct stat buf;

  switch(Type)
  {
    case SERIAL_DS2480:
    case PARALLEL_DS1481:
      /* Check to see if file exists */
      if (stat(FileName, &buf) != -1)
        return TRUE;
      break;

    default:
      break;
  }
 
  return FALSE;
}      

uchar EnterPassthru()
{
  TogglePassthru();
  Passthru = TRUE;

  return TRUE;
}

uchar ExitPassthru()
{
  int i = 0;

  Passthru = TRUE;

  while (Passthru && (i++ < 3))
  {
    if (Passthru)
      TogglePassthru();

    Passthru = !DS1481Present();
  }

  iBOverdriveOff(); 

  return !Passthru; 
}

void TogglePassthru()
{
  Send4();

  /* Wait for mode to change */
#ifdef USES_NAP
  nap(16);
#else
  usleep(16000);  
#endif
}

void FastSleep(ushort MSec)
{
   usleep(MSec * 1000);
}

uchar iBOverdriveOn()
{
  if (!DOWReset())
    return FALSE;
 
  /* Put all devices (that support overdrive) in overdrive mode */
  DOWByte(0x3C);

  return ToggleOverdrive() ? TRUE : ToggleOverdrive();
}

void iBOverdriveOff()
{
  if (ToggleOverdrive())
    ToggleOverdrive();

  return;
}

uchar SetAdapterType(uchar type,char *port)
{
  uchar RetCode;
  int   i;
  
  RetCode = TRUE;

  switch (type)
  {
    case PARALLEL_DS1481:
      /* Set Parallel port number to zero. */
      PortNum = 0;
    case SERIAL_DS2480:
      i = 0;
      while ((port[i] != 0x00) && (i < (MAXFILENAMELEN-1)))
      {
        FileName[i] = port[i];
        i++;
      }
      if (port[i] == 0x00)
      {
        FileName[i] = 0x00;
        Type        = type;
      }
      else
        RetCode = FALSE; 
      break;

    default:
      RetCode = FALSE;
      break;
  }

  return RetCode;
}

uchar SetAdapterSpeed(ulong speed)
{
  switch (Type)
  {
    case SERIAL_DS2480:
      if (DS2480SetSpeed(FileDes,speed) >= 0)
      {
        return TRUE;
      }
      else
      {
        return FALSE;
      }
      break;

    default:
      break;
  }
  
  return TRUE;
}

uchar SetAdapter5VTime(uchar time)
{
  switch (Type)
  {
    case SERIAL_DS2480:
      if (DS2480SetParm(FileDes,PARMSEL_5VPULSE,time) >= 0)
      {
        return TRUE;
      }
      else
      {
        return FALSE;
      }
      break;

    default:
      break;
  }
  
  return TRUE;
}

uchar Adapter5VPrime()
{
  DS2480BitPrime();
  
  return TRUE;
}

uchar Adapter5VCancel()
{
  switch (Type)
  {
    case SERIAL_DS2480:
      if (DS2480StrongPUOff(FileDes) >= 0)
      {
        return TRUE;
      }
      else
      {
        return FALSE;
      }
      break;

    default:
      break;
  }
  
  return FALSE;
}

uchar Adapter5VActivate(ulong ustimeout)
{
  switch (Type)
  {
    case SERIAL_DS2480:
      if (DS2480StrongPUOn(FileDes) >= 0)
      {
        usleep(ustimeout);
        /* Turn off just to make sure */
        DS2480StrongPUOff(FileDes);
        return TRUE;
      }
      else
      {
        return FALSE;
      }
      break;

    default:
      break;
  }
  
  return TRUE;
}

uchar iBSetup(uchar pn)             
{
  struct stat buf;

  SetupOk = FALSE;

  switch(Type)
  {
    case SERIAL_DS2480:
      /* Check to see if file exists */
      if (stat(FileName, &buf) != -1)
        SetupOk = FailNext = AccessGood = TRUE;
      break;

    case PARALLEL_DS1481:
      if ((pn > 0) && (pn < 4))
      {
        /* Set the port number */
        PortNum = pn;
        /* Check to see if file exists */
        if (stat("/dev/dow0", &buf) != -1)
          SetupOk = FailNext = AccessGood = TRUE;
      }
      break;

    default:
      break;
  }
 
  return SetupOk;                       
}

/*
 *  Find next part on DOW bus 
 */
uchar iBNext()                                          
{
   uchar tr;

   if (SetupOk)
   {
      /* See if last search found last button */
      if (FailNext)                 
      {
         FailNext = FALSE; 
         /* Reset next function  */
         l0 = 0;                                    
      } 
      else if ((tr = RomSearch()) != 0) 
      {
         /* See if we should force failure */
         if (tr == 2) 
            FailNext = TRUE;                 
        
         return TRUE;
      }
   }

   return FALSE;                                
}

/* 
 *  Find first DOW part on specified port  
 */
uchar iBFirst()                     
{
   /* Don't force a failure here  */
   FailNext = FALSE;              
   /* Point Rom Search algorithm to the top */
   l0 = 0;                         

   /* Go look for the first DOW part on the bus */
   return iBNext();             
}

/*
 *  "Strong" access
 */
uchar iBStrongAccess()               
{
   uchar i, j;             
   uchar mask = 0x80;
   uchar byte;
   uchar bit;
   uchar bytesend[24]; 

   /* Assume failure  */
   AccessGood = FALSE;                                   

   /* Send reset pulse */
   if (DOWReset())                                      
   {
      /* ROM search command byte */
      DOWByte(0xF0);                             

      switch (Type)
      {
        case SERIAL_DS2480:
          /* Setup the 24 byte search block */
          for (i = 0;i < 24;i++)
            bytesend[i] = 0xFF;
          byte = 0;
          bit = 2;
          for (i = 0;i < 8;i++)
          {
            for (j = 0;j < 8;j++)
            {
              if (((RomDta[i] >> j) & 1) != 1)
                bytesend[byte] &= ~(mask >> (7 - bit));
              bit += 3;
              if (bit >= 8)
                byte++;
              bit = (bit) % 8;
            }
          }
          /* Send the block. */
          DOWBlock(bytesend,24);
          /* Check the block, make sure we talked to a part.
             If we get two ones on the last triplet, we definitely
             have lost the part.
          */
          if ((bytesend[23] & 0x60) == 0x60)
            return FALSE;
          break;
        case PARALLEL_DS1481:
          /* Byte loop */
          for (i = 0; i < 8; i++)          
          {
            /* Bit loop */
            for (j = 0; j < 8; j++)      
            {
              /* Send two read time slots */
              if (((DOWBit(TRUE) << 1) | DOWBit(TRUE)) == 3)                
                return FALSE;         

              /* Send write time slot */
              DOWBit((uchar) ((RomDta[i] >> j) & 1));
            }      
          }
          break;
        default:
          break;
      }
      
      /* Success if we made it through all bits */
      AccessGood = TRUE;      
   }

   return AccessGood;  
}

/*
 *  Reset buttons on port.
 */
uchar iBReset()
{
  return(DOWReset());
}

/* 
 *  Xmit data to buttons on the port 
 */
uchar iBDataByte(uchar br)                      
{
  return (SetupOk && AccessGood) ? DOWByte(br) : br;
}

uchar iBDataBit(uchar br)
{
  return (SetupOk && AccessGood) ? DOWBit(br) : br;
}

uchar iBDataBlock(uchar *barr, int length)
{
  return (SetupOk && AccessGood) ? DOWBlock(barr,length) : barr[0];
}

uchar iBStreamRS(uchar *arr,int length,int run)
{
  int i;
  int AccessGood = FALSE;
  uchar accessarray[9];
  
  switch (Type)
  {
    case SERIAL_DS2480:
      AccessGood = DS2480StreamRS(FileDes,RomDta,arr,length,run);
      break;
      
    default:
      if (iBReset())
      {
        AccessGood = TRUE;
  
        /* Set up block for transmit */
        accessarray[0] = 0x55;
        for (i = 1;i < 9;i++)
          accessarray[i] = RomDta[i-1];
        /* Transmit block. */    
        iBDataBlock(accessarray,9);
        iBDataBlock(arr,length);
        if (run == 1)
        {
          SetAdapter5VTime(PARMSET_infinite);
          Adapter5VPrime();
          AccessGood = (iBDataBit(1)?0:1);
        }
      }
      break;
  }

  return AccessGood;  
}

uchar iBStream(uchar *arr,int length)
{
  int i;
  int AccessGood = FALSE;
  uchar accessarray[9];
  
  switch (Type)
  {
    case SERIAL_DS2480:
      AccessGood = DS2480Stream(FileDes,RomDta,arr,length);
      break;
      
    default:
      if (iBReset())
      {
        AccessGood = TRUE;
  
        /* Set up block for transmit */
        accessarray[0] = 0x55;
        for (i = 1;i < 9;i++)
          accessarray[i] = RomDta[i-1];
        /* Transmit block. */    
        iBDataBlock(accessarray,9);
        iBDataBlock(arr,length);
      }
      break;
  }

  return AccessGood;  
}

uchar iBAccess()
{
  int i;
  uchar accessarray[9];
  
  AccessGood = FALSE;

  switch (Type)
  {
    case SERIAL_DS2480:
      AccessGood = DS2480Access(FileDes,RomDta);
      break;
      
    default:
      if (iBReset())
      {
        AccessGood = TRUE;
  
        /* Set up block for transmit */
        accessarray[0] = 0x55;
        for (i = 1;i < 9;i++)
          accessarray[i] = RomDta[i-1];
        /* Transmit block. */    
        iBDataBlock(accessarray,9);
      }
      break;
  }
  
  return AccessGood;
}

uchar iBFastAccess()
{
   return iBAccess();
}

uchar gndtest()
{
   if (SetupOk)
   {
      /* Put parts on bus in known state */
      DOWReset();                        
      /* Return result of write 1 time slot */
      return DOWBit(TRUE);
   }

   return FALSE;
}

uchar *iBROMData()
{
   /* Return pointer to ROM Data buffer  */
   return RomDta;                     
}

/*****************************************************************************
 *
 *    This function performs a ROM search and finds one part on the DOW bus
 * per call.
 *
 *     Return values : 0 => No parts on bus or bus error
 *                     1 => A part was found and more are out there
 *                     2 => A part was found and it was last one on the bus
 *
 */
int RomSearch() 
{
  uchar i = 0, x = 0, ld = l0;
  uchar RomBit, Mask;
  uchar SearchBuf[16];

  switch (Type)
  {
    case SERIAL_DS2480:
      /* Set up 16 bytes of ROMID data */
      for (i = 0;i < 16;i++) SearchBuf[i] = 0x00;
      /* If we're not doing a first. */
      if (l0 != 0)
      {
        i = 0;
        while (i++ < 64)
        {
          if (i >= ld)
          {
            if (i == ld)
              RomBit = 1;
            else
              RomBit = 0;
          }
          else
            RomBit = (RomDta[(i - 1) / 8] >> ((i - 1) % 8)) & 0x01;
          SearchBuf[(i - 1) / 4] |= RomBit << (((i-1)%4)*2+1);
        }
      }

      /* Returns false if Presence Pulse not detected. */
      if (DS2480Search(FileDes,SearchBuf) < 0)
        return(FALSE);

      /* Check for invalid search */
      if ((SearchBuf[15] & 0xC0) == 0xC0)
        x = 3;

      /* Copy into the RomDta buffer */
      for (i = 0;i < 8;i++) RomDta[i] = 0x00;
      i = 0;
      while (i++ < 64)
      {
        RomBit = (SearchBuf[(i - 1) / 4] >> (((i - 1) % 4) * 2 + 1)) & 0x01;
        RomDta[(i - 1) / 8] |= (RomBit << (i - 1)%8);
      }
      /* Find the last discrepancy */
      i = 64;
      while (i > 0)
      {
        if ( SearchBuf[(i - 1) / 4] & (0x01 << (((i-1)%4) * 2)) &&
             !(SearchBuf[(i - 1) / 4] & (0x01 << (((i-1)%4) * 2 + 1))) )
        {
          l0 = i;
          i = 1;
        }
        i--;
      }
      break;
    case PARALLEL_DS1481:
      /* Reset DOW bus */
      if (DOWReset())     
      {
        /* Send search command */
        DOWByte(0xF0);   
      }
      else
        return FALSE;

      /* While not done and bus error */
      while ((i++ < 64) && (x < 3))            
      {
        /* Get bit mask */
        Mask = 1 << ((i - 1) % 8);           

        RomBit = RomDta[(i - 1) >> 3] & Mask ? TRUE : FALSE; 

        /* Send first read time slot */
        x = DOWBit(TRUE) << 1; 
        /* Send second read time slot */
        x |= DOWBit(TRUE);      
  
        /* Is there a disagreement in this bit position */
        if (!x)               
        {
          /* Stay on old path or pick a new one ?  */
          if (i >= ld)              
            RomBit = (i == ld);   /* Send write 1 if at position of ld  */

          /* Save this value as temp last disagreement  */
          if (!RomBit)         
            l0 = i;
        } 
        else 
          RomBit = (x & 1) ^ 1;    /* Get lsb of x and flip it  */

        /* Send write time slot */
        DOWBit(RomBit);                               

        if (RomBit)              
        {
          /* Set bit in Rom Data byte */
          RomDta[(i - 1) >> 3] |= Mask;          
        }
        else
        {
          /* Clear bit in ROM data byte */
          RomDta[(i - 1) >> 3] &= (Mask ^ 0xFF); 
        }
      }
      break;
    default:
      break;
  }
 
  return (x == 3) ? 0 : 1 + (ld == l0);    
}

int DOWReset()
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        if (DS2480Reset(FileDes) == 1)
          return(1);
        else
          return(0);
        break;
      case PARALLEL_DS1481:
        /* Assign command byte */
        DS1481Comm.Command = DO_RESET;                 
        /* Set port number */
        DS1481Comm.Xfer = PortNum;

        /* Send info to DOW driver */
        write(FileDes, &DS1481Comm.Command, 2);      
        /* Get result */
        read(FileDes, &DS1481Comm.Xfer, 1);         

        /* Return result of reset operation */
        return DS1481Comm.Xfer;
        break;
      default:
        return FALSE;
        break;
    }
  }
  /* Return FALSE if process did not have control of bus */
  return FALSE;     
}

/*
 *  Send bit to 1-wire bus
 */
int DOWBit(uchar BitVal)                         
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        if (DS2480Bit(FileDes,BitVal) == 1)
          return(1);
        else
          return(0);
        break;
      case PARALLEL_DS1481:
        /* Assign command byte */
        DS1481Comm.Command = DO_BIT;               
        /* Byte to send to 1-wire bus */ 
        DS1481Comm.Xfer = BitVal;               
        /* Send packet to driver */
        write(FileDes, &DS1481Comm.Command, 2);      
        /* Get result of write */
        read(FileDes, &DS1481Comm.Xfer, 1);            

        return DS1481Comm.Xfer; 
        break;
      default:
        return(FALSE);
        break;
    }  
  }
  return BitVal;
}

/*
 *  Send byte to 1-wire bus 
 */   
int DOWByte(uchar ByteVal)                      
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        return(DS2480Byte(FileDes,ByteVal));
        break;

      case PARALLEL_DS1481:
        /* Assign command byte */
        DS1481Comm.Command = DO_BYTE;             
        /* Byte to send to 1-wire bus */
        DS1481Comm.Xfer = ByteVal;              
        /* Send info to DOW driver */
        write(FileDes, &DS1481Comm.Command, 2);  
        /* Get result of write */
        read(FileDes, &DS1481Comm.Xfer, 1);     

        return DS1481Comm.Xfer;  
        break;

      default:
        return FALSE;
        break;
    }
  } 

  return ByteVal;
}

/* Send a stream of bytes to the port. */
int DOWBlock(uchar *barr, int length)
{
  int i;
  
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        for (i = 0;i < length;i += SERIAL_DS2480_BLKSIZE)
        {
          if ((length - i) < SERIAL_DS2480_BLKSIZE)
            DS2480Block(FileDes,(barr + i),length - i);
          else
            DS2480Block(FileDes,(barr + i),SERIAL_DS2480_BLKSIZE);
        }
        return TRUE;
        break;

      case PARALLEL_DS1481:
        for (i = 0;i < length;i++)
          barr[i] = DOWByte(barr[i]);
        return TRUE;
        break;

      default:
        return FALSE;
        break;
    }
  }  
  
  /* return(barr[0]); ????????????? */
  return TRUE;
}

int ToggleOverdrive()
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        return(DS2480ToggleOverdrive(FileDes));
        break;
      case PARALLEL_DS1481:
        /* Assign command byte */
        DS1481Comm.Command = TOGGLE_OVERDRIVE;                 
        DS1481Comm.Xfer    = 0;

        /* Send info to DOW driver */
        write(FileDes, &DS1481Comm.Command, 2);      
        /* Get result */
        read(FileDes, &DS1481Comm.Xfer, 1);         

        /* Return result of reset operation */
        return DS1481Comm.Xfer;           
        break;

      default:
        return FALSE;
        break;
    }
  }

  return FALSE;     
}

int Send4()
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        return TRUE;
        break;

      case PARALLEL_DS1481:  
        /* Assign command byte */
        DS1481Comm.Command = TOGGLE_PASSTHRU;                 
        DS1481Comm.Xfer    = 0;

        /* Send info to DOW driver */
        write(FileDes, &DS1481Comm.Command, 2);      
        /* Get result */
        read(FileDes, &DS1481Comm.Xfer, 1);         

        /* Return result of reset operation */
        return DS1481Comm.Xfer;           
        break;

      default:
        return FALSE;
        break;
    }
  }

  return FALSE;     
}

int DS1481Present()
{
  if (AuthFlag)
  {
    switch(Type)
    {
      case SERIAL_DS2480:
        return(DS2480Present(FileDes));
        break;

      case PARALLEL_DS1481:
        /* Assign command byte */
        DS1481Comm.Command = CHECK_BRICK;                 
        DS1481Comm.Xfer    = 0;

        /* Send info to DOW driver */
        write(FileDes, &DS1481Comm.Command, 2);      
        /* Get result */
        read(FileDes, &DS1481Comm.Xfer, 1);         

        /* Return result of reset operation */
        return DS1481Comm.Xfer;           
        break;

      default:
        return FALSE;
        break;
    }
  }

  return FALSE;     
}
