//////////////////////////////////////////////////////////////////////////
//
//         CiBApi.h (Include file for the Micro-In-A-Can API)  
//
//                          { Ver 0.81 12/09/96 }
//
//////////////////////////////////////////////////////////////////////////

#ifndef _INC_WINDOWS
#ifndef CIB_UNIX
#include "windows.h"
#endif
#endif

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

#ifdef WIN32
#define DLLEXPORT __stdcall
#endif

#ifdef WIN16
#define DLLEXPORT far _export pascal
#endif

#include "cibtypes.h"
#include "adapter.h"

/*
 *    API defines
 */
#define PIN_TO_ERASE  1
#define PIN_TO_CREATE 2

/*
 *    Max length defines
 */
#define MAX_PIN_LEN    8        // Maximum PIN length
#define MAX_NAME_LEN   16       // Maximum group name length
#define MAX_PACKET_LEN 128      // Maximum data packet length
#define MAX_OBJ_LEN    128      // Maximum object length
#define MAX_NUM_LEN    128      // Largest prime number generated
#define MAX_CIB_NUM    16       // Maximum number of CiBs on system
#define MAX_BUF_SIZE   255
#define MAX_SEND       255      // Maximum send buffer size
#define MAX_RECEIVE    255      // Maximum receive buffer size

/*
 *    Object attribute bits
 */
#define LOCKED         1
#define PRIVATE        2
#define DESTRUCT       4
#define CIB_CREATE     128

#ifndef _SYS_TYPES_H
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;
#endif

#ifdef _BORLANDC_
typedef BOOL FAR *LPBOOL;      
#endif

/*
 *   Structure definitions
 */
typedef struct _CMDPACKET
{
   BYTE Len;
   BYTE CmdByte;
   BYTE GroupID;
   BYTE CmdData[MAX_SEND]; 
}
CMDPACKET, *PCMDPACKET, NEAR *NPCMDPACKET, FAR *LPCMDPACKET;

typedef struct _RETPACKET      // Firmware return packet.
{
   BYTE CSB;
   BYTE GroupID;
   BYTE DataLen;
   BYTE CmdData[MAX_RECEIVE];
}
RETPACKET, *PRETPACKET, NEAR *NPRETPACKET, FAR *LPRETPACKET;

typedef struct _CIBPIN
{
   BYTE Len;
   BYTE PINData[MAX_PIN_LEN];
}
CIBPIN, *PCIBPIN, NEAR *NPCIBPIN, FAR *LPCIBPIN;

typedef struct _CIBNAME
{
   BYTE Len;
   BYTE NameData[MAX_PACKET_LEN];
}
CIBNAME, *PCIBNAME, NEAR *NPCIBNAME, FAR *LPCIBNAME;

typedef struct _CIBOBJ
{
   BYTE Attr;                 // Object attribute byte (permissions) 
   BYTE Type;                 // Object type specifier
   BYTE Len;
   BYTE ObjData[MAX_OBJ_LEN];
}
CIBOBJ, *PCIBOBJ, NEAR *NPCIBOBJ, FAR *LPCIBOBJ;

typedef struct _BIGNUM
{
   BYTE Len;
   BYTE NumArr[MAX_NUM_LEN];
}
BIGNUM, *PBIGNUM, NEAR *NPBIGNUM, FAR *LPBIGNUM;

typedef struct _CIBINFO
{
   BYTE GroupNum;             // Number of groups in device
   BYTE CiBFlags;            
}
CIBINFO, *PCIBINFO, NEAR *NPCIBINFO, FAR *LPCIBINFO; 

typedef struct _COMMPACKET
{
   BYTE Len;
   BYTE CmdData[MAX_PACKET_LEN];
}
COMMPACKET, *PCOMMPACKET, NEAR *NPCOMMPACKET, FAR *LPCOMMPACKET;

/*
 *     Object type bytes
 */
#define CIB_OUTPUT_DATA      0   // Auto object types
#define CIB_WORKING_REGISTER 1
#define CIB_ROM_DATA         2
#define CIB_RANDOM_FILL      3
#define CIB_MODULUS          32  // User objects
#define CIB_EXPONENT         33
#define CIB_MONEY            34
#define CIB_COUNTER          35
#define CIB_SCRIPT           36
#define CIB_CLOCK_OFFSET     37
#define CIB_SALT             38
#define CIB_CONFIG_DATA      39
#define CIB_INPUT_DATA       40
#define CIB_DESTRUCTOR       41

/*
 *    Command byte values
 */
#define SET_COMMON_PIN            1
#define MASTER_ERASE              2
#define CREATE_TRANSACTION_GROUP  3
#define SET_GROUP_PIN             4
#define CREATE_OBJECT             5
#define LOCK_OBJECT               6
#define PRIVATIZE_OBJECT          7
#define DESTRUCTIBLE_OBJECT       8
#define LOCK_CIB                  9
#define LOCK_GROUP                10
#define INVOKE_SCRIPT             11
#define READ_OBJECT               12
#define WRITE_OBJECT              13
#define READ_GROUP_NAME           14
#define DELETE_GROUP              15
#define GET_GROUP_ID              16
#define READ_CONFIG_INFO          17
#define READ_REAL_TIME_CLOCK      18
#define READ_TRUE_TIME_CLOCK      19
#define CHECK_GROUP_CRC           20
#define GET_RANDOM_DATA           21
#define GET_VER_STRING            22
#define READ_FREE_RAM             23
#define CHANGE_GROUP_NAME         24
#define DISABLE_KS_GEN            25
#define GEN_RSA_KS                26
#define GEN_RSA_MOD_AND_EXP       27
#define GEN_RSA_KS_NO_PRIVATE     28
#define GEN_PRIME                 29
#define SEND_COMM_COMMAND         30

/*
 *    Comm Level command bytes
 */
#define ECHO_MESSAGE              129
#define RESET_CMD_INTERPRETER     131
#define DUMP_REGS                 132
#define SET_MAX_BLOCK_SIZE        133
#define RAM_TEST                  134
#define ROM_TEST                  135

/*
 *    Command interpreter error codes
 */
#define CMD_OK                    0
#define ERR_BAD_COMMON_PIN        0x81
#define ERR_BAD_GROUP_PIN         0x82
#define ERR_BAD_PIN_LENGTH        0x83
#define ERR_BAD_NAME_LENGTH       0x85
#define ERR_INSUFFICIENT_RAM      0x86
#define ERR_CIB_LOCKED            0x87
#define ERR_CIB_NOT_LOCKED        0x88
#define ERR_GROUP_LOCKED          0x89
#define ERR_BAD_OBJECT_TYPE       0x8A
#define ERR_BAD_OBJECT_ATTR       0x8B
#define ERR_BAD_SIZE              0x8C
#define ERR_BAD_GROUP_ID          0x8D
#define ERR_BAD_OBJECT_ID         0x8E
#define ERR_INSUFFICIENT_FUNDS    0x8F
#define ERR_OBJECT_LOCKED         0x90
#define ERR_OBJECT_PRIVATE        0x91
#define ERR_OBJECT_DESTRUCTED     0x92
#define ERR_MAX_GROUPS            0x93
#define ERR_MAX_OBJECTS           0x94
#define ERR_NOT_SCRIPT_ID         0x95
#define ERR_OPEN_GROUP            0x96
#define ERR_BAD_GROUP_CRC         0x97
#define ERR_GROUP_NOT_FOUND       0x98
#define ERR_NO_KEY_GENERATION     0x99
#define ERR_BAD_MODULUS_ID        0x9A
#define ERR_BAD_EXPONENT_ID       0x9B
#define ERR_NOT_CIB_CREATED       0x9C
#define ERR_EXP_NOT_REL_PRIME     0x9D

/*
 *     Script interpreter error codes
 */
#define ERR_OPERATOR_NOT_EXPECTED 0xC0
#define ERR_EOS_EXPECTED          0xC1
#define ERR_BAD_ID                0xC2
#define ERR_NOT_COMPOSITE         0xC3
#define ERR_UNEXPECTED_END        0xC4
#define ERR_NOT_AN_OPERATOR       0xC5
#define ERR_BAD_TYPE              0xC6
#define ERR_MEMBER_NOT_FOUND      0xC7
#define ERR_BAD_COMPARE           0xC8
#define ERR_BAD_ADDITION          0xC9
#define ERR_BAD_SUBTRACTION       0xCA
#define ERR_SIZE                  0xCB
#define ERR_NOT_MONEY             0xCC
#define ERR_NOT_RSA_EXPONENT      0xCD
#define ERR_NOT_RSA_MODULUS       0xCE
#define ERR_MOD_OP_EXPECTED       0xCF
#define ERR_DESTRUCTED_OBJECT     0xD0
#define ERR_INACTIVE              0xD1

/*
 *     API communication error codes
 */
#define ERR_NO_CIBS_FOUND        0xF000
#define ERR_BAD_CIB_ROM          0xF100
#define ERR_CIB_NOT_FOUND        0xF200
#define ERR_ADAPTER_NOT_FOUND    0xF300
#define ERR_COMM_FAILURE         0xF400

BOOL DispatchCommand(LPCMDPACKET lpCP, LPRETPACKET lpRP);
void SetCiBError(WORD);
WORD GetCiBErrorLocal();

#ifdef __cplusplus
extern "C" {
#endif

/*
 *      API functions
 */
void   DLLEXPORT SetCiBSearchParams(BYTE, char **);
LPBYTE DLLEXPORT FindCiBs(LPBYTE);
BOOL   DLLEXPORT SelectCiB(LPBYTE);
WORD   DLLEXPORT GetCiBError();
BOOL   DLLEXPORT ReadFirmwareVersionID(LPCIBNAME, LPRETPACKET);
BOOL   DLLEXPORT ReadRandomBytes(BYTE, LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT SetCommonPIN(LPCIBPIN, LPCIBPIN, BYTE, LPRETPACKET);
BOOL   DLLEXPORT MasterErase(LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT CreateTransactionGroup(LPCIBPIN, LPCIBNAME, LPCIBPIN, BYTE,
                                        LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT ReadGroupName(BYTE, LPCIBNAME, LPRETPACKET);
BOOL   DLLEXPORT CheckGroupCRC(BYTE, LPRETPACKET);
BOOL   DLLEXPORT ChangeGroupName(BYTE, LPCIBPIN, LPCIBNAME, LPRETPACKET);
BOOL   DLLEXPORT CreateCiBObject(BYTE, LPCIBPIN, LPCIBOBJ, 
                                 LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT ReadCiBObject(BYTE, LPCIBPIN, BYTE, LPCIBOBJ, LPRETPACKET);
BOOL   DLLEXPORT WriteCiBObject(BYTE, LPCIBPIN, BYTE, LPCIBOBJ, LPRETPACKET);
BOOL   DLLEXPORT SetCiBObjectAttr(BYTE, LPCIBPIN, BYTE, BYTE, LPRETPACKET);
BOOL   DLLEXPORT SetGroupPIN(BYTE, LPCIBPIN, LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT LockGroup(BYTE, LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT LockCiB(LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT DeleteGroup(BYTE, LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT GetGroupID(LPCIBNAME, LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT GetCiBConfiguration(LPCIBINFO, LPRETPACKET);
BOOL   DLLEXPORT ReadRealTimeClock(LPDWORD, LPRETPACKET);
BOOL   DLLEXPORT ReadTrueTimeClock(BYTE, LPCIBPIN, BYTE, LPDWORD, LPRETPACKET);
BOOL   DLLEXPORT ReadFreeRAM(LPWORD, LPRETPACKET);
BOOL   DLLEXPORT InvokeScript(BYTE, LPCIBPIN, BYTE, WORD, LPRETPACKET);
BOOL   DLLEXPORT GenerateRSAKeySet(BYTE, LPCIBPIN, BYTE, LPBYTE, LPBYTE,
                                   LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT GenerateRSAKeySetNP(BYTE, LPCIBPIN, BYTE, LPBYTE, LPBYTE,
				     LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT GenerateRSAModAndExp(BYTE, LPCIBPIN, BYTE, BYTE, LPBYTE, 
				      LPBYTE, LPRETPACKET);
BOOL   DLLEXPORT DisableKeySetGeneration(LPCIBPIN, LPRETPACKET);
BOOL   DLLEXPORT GeneratePrime(BYTE, LPCIBPIN, BYTE, LPBIGNUM, LPRETPACKET);
BOOL   DLLEXPORT SendCommCommand(BYTE, LPCOMMPACKET, LPRETPACKET);

/*
 *  Low level comm functions
 */
BYTE DLLEXPORT SearchPorts(LPBYTE, LPBYTE, LPBOOL, LPIBHOST);
WORD DLLEXPORT SendCiBMessage(LPBYTE, WORD, WORD, PIBADDR);
WORD DLLEXPORT RecvCiBMessage(LPBYTE, LPWORD, PIBADDR);

#ifdef __cplusplus
}
#endif
