/****************************************************************************
 *
 *
 * CiBCyrpt.c                                    { Ver 0.10 12/13/96 }
 * (DWL)
 *
 ***************************************************************************/

#include <memory.h>
#include <malloc.h>

#include "cibtypes.h"
#include "cibcrypt.h"
#include "crypteng.h"

BOOL CiBCommOk = FALSE;
BYTE GpID;

/****************************************************************************
 *
 *    Find attached CiBs. If we have 1 (and only 1) CiB, Find the group ID
 * for the "Dallas Primary" group and save it in GpID.
 *
 */
WORD DLLEXPORT InitializeCiB(BYTE SPCount, char *DevNames[])
{
   LPBYTE    lpROMs;
   BYTE      CiBCount;
   WORD      ErrCode;
   CIBNAME   GpName;
   RETPACKET RP;

   if (SPCount)
      SetCiBSearchParams(SPCount, DevNames); 

   /* Search adapters for attached CiBs */
   lpROMs = FindCiBs(&CiBCount);

   /* Only want to worry about 1 device for now */
   if (CiBCount > 1)
      return MULTIPLE_CIBS;

   if (lpROMs)
   {
      /* Select the 1st and only CiB */
      SelectCiB(lpROMs);

      GpName.Len = strlen("Dallas Primary");
      strcpy(GpName.NameData, "Dallas Primary");

      /* Search for E-Mail Demo group */
      if (!GetGroupID(&GpName, &GpID, &RP))
         return GROUP_NOT_FOUND;
   }

   ErrCode = GetCiBError();

   if (!ErrCode)
      CiBCommOk = TRUE;

   return ErrCode;
}

/****************************************************************************
 *
 *    Reads the public key from the E-Mail Demo group.
 *
 */
WORD DLLEXPORT ReadPublicKey(LPWORD lpModLen, 
                             LPBYTE lpMod, 
                             LPWORD lpExpLen, 
                             LPBYTE lpExp,
                             BYTE   PINLen,
                             LPBYTE lpPIN)
{
   CIBPIN    GpPIN;
   CIBOBJ    CiBMod, CiBExp;
   WORD      ErrCode;
   RETPACKET RP;

   /* Check comm state */
   if (CiBCommOk)
   {
      /* Initialize group PIN structure */
      GpPIN.Len = PINLen;
      memcpy(GpPIN.PINData, lpPIN, PINLen);

      /* Read modulus */
      if (ReadCiBObject(GpID, &GpPIN, RSA_MOD, &CiBMod, &RP))
      {
         /* Read public exponent */
         if (ReadCiBObject(GpID, &GpPIN, RSA_EXP_PUB, &CiBExp, &RP))
         {
            /* Transfer modulus info */
            *lpModLen = CiBMod.Len;
            memcpy(lpMod, CiBMod.ObjData, CiBMod.Len);
            /* Transfer exponent info */
            *lpExpLen = CiBExp.Len;
            memcpy(lpExp, CiBExp.ObjData, CiBExp.Len);
         }
      }
   }
   else
      return COMM_STATE_BAD;

   ErrCode = GetCiBError();

   if (ErrCode && (ErrCode != ERR_BAD_GROUP_PIN))
      CiBCommOk = FALSE;

   return ErrCode;
}

/****************************************************************************
 *
 *   Encrypt message passed in lpMsg buffer. The buffer should be large 
 * enough to grow by up to 16 bytes. The message is encrypted using safer 
 * sk128. The safer key is generated by requesting 16 bytes of random data 
 * from the attached Crypto iButton. Once the message is encrypted the safer
 * key is encrypted by writing the public key of the recipient to the open
 * modulus and exponent objects of the CiB and invoking the encryption 
 * script.
 *
 */
WORD DLLEXPORT EncryptMessage(LPDWORD lpMsgLen,     
                              LPBYTE  lpMsg,         /* Message buffer */
                              LPBYTE  lpEncKeyLen,
                              LPBYTE  lpEncKey,      /* Encrypted key buffer */
                              BYTE    ModLen,
                              LPBYTE  lpMod,         /* Recipients exponent */
                              BYTE    ExpLen,
                              LPBYTE  lpExp,         /* Recipients modulus */
                              BYTE    PINLen,
                              LPBYTE  lpPIN)
{
   WORD      ErrCode;
   BYTE      KeyBuff[MOD_LEN];
   LPBYTE    TempBuff;
   CIBPIN    GpPIN;
   CIBOBJ    CiBKeyData, CiBMod, CiBExp;
   DWORD     ChkSum, i;
   RETPACKET RP;

   /* Check comm state */
   if (CiBCommOk)
   {
      memset(KeyBuff, 0x55, MOD_LEN);

      /* Get random bytes to be used as safer key */
      if (ReadRandomBytes(KEY_LEN, KeyBuff, &RP))
      {
         /* Duplicate key for integrity check after decryption */
         memcpy(&KeyBuff[KEY_LEN], KeyBuff, KEY_LEN); 

         /* Initialize PIN structure */
         GpPIN.Len = PINLen;
         memcpy(GpPIN.PINData, lpPIN, PINLen);

         /* Initialize input data object */
         CiBKeyData.Len = MOD_LEN;
         memcpy(CiBKeyData.ObjData, KeyBuff, MOD_LEN);

         /* Initialize recipients modulus object structure */
         CiBMod.Len = (BYTE) ModLen;
         memcpy(CiBMod.ObjData, lpMod, ModLen);

         /* Initialize recipients public exponent object structure */
         CiBExp.Len = (BYTE) ExpLen;
         memcpy(CiBExp.ObjData, lpExp, ExpLen);

         /* 
          *    Write key data, other guys and other guys modulus to 
          * respective objects 
          */
         if (WriteCiBObject(GpID,&GpPIN,INPUT_DATA,&CiBKeyData,&RP) &&
             WriteCiBObject(GpID,&GpPIN,RECIPIENT_MOD,&CiBMod,&RP) &&
             WriteCiBObject(GpID,&GpPIN,RECIPIENT_EXP_PUB,&CiBExp,&RP))
         {
            /* Encrypt the symmetric key */
            if (InvokeScript(GpID, 
                             &GpPIN, 
                             ENCRYPT_SCRIPT, 
                             ENCRYPT_SCRIPT_RUN_TIME, 
                             &RP))
            {
               /* Read encrypted key */
               if (ReadCiBObject(GpID, &GpPIN, OUTPUT_DATA, &CiBKeyData, &RP))
               {
                  /* Transfer encrypted symmetric key data */
                  *lpEncKeyLen = CiBKeyData.Len;
                  memcpy(lpEncKey, CiBKeyData.ObjData, CiBKeyData.Len);

                  /*
                   *    Done screwing around with the CiB. Encrypt the 
                   * message using the safer key in KeyBuff.
                   */

                  ChkSum = 0;
                  for (i = 0; i < *lpMsgLen; i++)
                     ChkSum += lpMsg[i];

                  /* Append checksum */
                  memcpy(&lpMsg[*lpMsgLen+(8 - *lpMsgLen%8)], 
                         (LPBYTE) &ChkSum, 
                         sizeof(DWORD));

                  /* Append original length */
                  memcpy(&lpMsg[*lpMsgLen+(8 - *lpMsgLen%8)+sizeof(DWORD)], 
                               (LPBYTE) lpMsgLen, 
                               sizeof(DWORD));

                  /* Compute new message length */
                  *lpMsgLen += (8 -  *lpMsgLen%BLOCK_LEN) + BLOCK_LEN;

                  TempBuff = (LPBYTE) malloc(*lpMsgLen * sizeof(BYTE));
                  memcpy(TempBuff, lpMsg, *lpMsgLen);
                  /* Encrypt caller's message */
                  Encrypt(TempBuff, lpMsg, (LPWORD) KeyBuff, *lpMsgLen);

                  free(TempBuff);
               }
            }
         }
      }
   }
   else
      return COMM_STATE_BAD;

   ErrCode = GetCiBError();

   if (ErrCode && (ErrCode != ERR_BAD_GROUP_PIN))
      CiBCommOk = FALSE;

   return ErrCode;
}


/****************************************************************************
 * 
 *    Same as EncryptMessage except that is used the same key set for 
 * encryption. This uses the script in object Id 9 for encryption.
 *
 */
WORD DLLEXPORT EncryptMessageSamePub(LPDWORD lpMsgLen,     
                                     LPBYTE  lpMsg,   
                                     LPBYTE  lpEncKeyLen,
                                     LPBYTE  lpEncKey, 
                                     BYTE    PINLen,
                                     LPBYTE  lpPIN)
{
   WORD      ErrCode;
   BYTE      KeyBuff[MOD_LEN];
   LPBYTE    TempBuff;
   CIBPIN    GpPIN;
   CIBOBJ    CiBKeyData; 
   DWORD     ChkSum, i;
   RETPACKET RP;

   /* Check comm state */
   if (CiBCommOk)
   {
      memset(KeyBuff, 0x55, MOD_LEN);

      /* Get random bytes to be used as safer key */
      if (ReadRandomBytes(KEY_LEN, KeyBuff, &RP))
      {
         /* Duplicate key for integrity check after decryption */
         memcpy(&KeyBuff[KEY_LEN], KeyBuff, KEY_LEN); 

         /* Initialize PIN structure */
         GpPIN.Len = PINLen;
         memcpy(GpPIN.PINData, lpPIN, PINLen);

         /* Initialize input data object */
         CiBKeyData.Len = MOD_LEN;
         memcpy(CiBKeyData.ObjData, KeyBuff, MOD_LEN);

         /* Write key data */
         if (WriteCiBObject(GpID,&GpPIN,INPUT_DATA,&CiBKeyData,&RP))
         {
            /* Encrypt the symmetric key */
            if (InvokeScript(GpID, 
                             &GpPIN, 
                             ENCRYPT_SAME_KEY, 
                             ENCRYPT_SCRIPT_RUN_TIME, 
                             &RP))
            {
               /* Read encrypted key */
               if (ReadCiBObject(GpID, &GpPIN, OUTPUT_DATA, &CiBKeyData, &RP))
               {
                  /* Transfer encrypted symmetric key data */
                  *lpEncKeyLen = CiBKeyData.Len;
                  memcpy(lpEncKey, CiBKeyData.ObjData, CiBKeyData.Len);

                  /*
                   *    Done screwing around with the CiB. Encrypt the 
                   * message using the safer key in KeyBuff.
                   */

                  ChkSum = 0;
                  for (i = 0; i < *lpMsgLen; i++)
                     ChkSum += lpMsg[i];

                  /* Append checksum */
                  memcpy(&lpMsg[*lpMsgLen+(8 - *lpMsgLen%8)], 
                         (LPBYTE) &ChkSum, 
                         sizeof(DWORD));

                  /* Append original length */
                  memcpy(&lpMsg[*lpMsgLen+(8 - *lpMsgLen%8)+sizeof(DWORD)], 
                         (LPBYTE) lpMsgLen, 
                         sizeof(DWORD));

                  /* Compute new message length */
                  *lpMsgLen += (8 -  *lpMsgLen%BLOCK_LEN) + BLOCK_LEN;

                  TempBuff = (LPBYTE) malloc(*lpMsgLen * sizeof(BYTE));
                  memcpy(TempBuff, lpMsg, *lpMsgLen);
                  /* Encrypt caller's message */
                  Encrypt(TempBuff, lpMsg, (LPWORD) KeyBuff, *lpMsgLen);

                  free(TempBuff);
               }
            }
         }
      }
   }
   else
      return COMM_STATE_BAD;

   ErrCode = GetCiBError();

   if (ErrCode && (ErrCode != ERR_BAD_GROUP_PIN))
      CiBCommOk = FALSE;

   return ErrCode;
}

/****************************************************************************
 *
 *    Decrypt message passed in the Msg buffer. The encrypted safer key is
 * passed in the EncKey buffer. This encrypted key is passed into the input
 * data object of the E-Mail Demo group and decrypted using the private 
 * exponent of same. The recovered safer key is then used to decrypt the 
 * message.
 *
 */
WORD DLLEXPORT DecryptMessage(LPDWORD lpMsgLen, 
                              LPBYTE  lpMsg, 
                              WORD    EncKeyLen, 
                              LPBYTE  lpEncKey,
                              BYTE    PINLen,
                              LPBYTE  lpPIN)
{
   WORD      ErrCode;
   DWORD     OrigLen, ChkSum, NewChk, i;
   LPBYTE    TempBuff;
   CIBOBJ    CiBKeyData;
   CIBPIN    GpPIN;
   RETPACKET RP;

   /* Check comm state */
   if (CiBCommOk)
   {
      /* Initialize PIN structure */
      GpPIN.Len = PINLen;
      memcpy(GpPIN.PINData, lpPIN, PINLen);

      /* Copy encrypted key data to CiB object */
      CiBKeyData.Len = (BYTE) EncKeyLen;
      memcpy(CiBKeyData.ObjData, lpEncKey, EncKeyLen);

      /* Write encrypted symmetric key to input data object */
      if (WriteCiBObject(GpID,&GpPIN,INPUT_DATA,&CiBKeyData,&RP))
      {
         if (InvokeScript(GpID, 
                          &GpPIN, 
                          DECRYPT_SCRIPT, 
                          DECRYPT_SCRIPT_RUN_TIME, 
                          &RP))
         {
            /* Read decrypted key */
            if (ReadCiBObject(GpID,&GpPIN,OUTPUT_DATA,&CiBKeyData,&RP))
            {
               /* Check integrity of decrypted key data */
               if (!memcmp(CiBKeyData.ObjData, 
                           &CiBKeyData.ObjData[KEY_LEN], 
                           KEY_LEN))
               {
                  TempBuff = (LPBYTE) malloc(*lpMsgLen * sizeof(BYTE));
                  memcpy(TempBuff, lpMsg, *lpMsgLen);
                  /* Decrypt message using recovered symmetric key */
                  Decrypt(TempBuff, 
                          lpMsg, 
                          (LPWORD) CiBKeyData.ObjData,
                          *lpMsgLen);

                  free(TempBuff);
                  
                  /* Recover original message length */
                  memcpy((LPBYTE) &OrigLen,
                          &lpMsg[(*lpMsgLen-sizeof(DWORD))], 
                         sizeof(DWORD));

                  if (OrigLen > *lpMsgLen)
                     return CHECK_SUM_BAD;

                  /* Recover original message checksum */
                  memcpy((LPBYTE) &ChkSum,
                          &lpMsg[(*lpMsgLen-2*sizeof(DWORD))], 
                          sizeof(DWORD));

                  NewChk = 0;
                  for (i = 0; i < OrigLen; i++)
                     NewChk += lpMsg[i];

                  if (NewChk == ChkSum)
                     *lpMsgLen = OrigLen;
                  else
                     return CHECK_SUM_BAD;
               }
               else
                  return KEY_DATA_BAD;
            }
         }
      }
   }
   else
      return COMM_STATE_BAD;

   ErrCode = GetCiBError();

   if (ErrCode && (ErrCode != ERR_BAD_GROUP_PIN))
      CiBCommOk = FALSE;

   return ErrCode;
}
