/****************************************************************************
 *
 * filecrypt.c                                        { Ver 0.01 11/2/96 }
 *
 *    This program is intended only as a demonstration on the use of the
 * Cryptographic iButton (CiB). 
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "filecrypt.h"
#include "cibcrypt.h"

char FileSig[]      = "CIBCRYPT";
char *DeviceNames[] = { "/dev/ttya" };

BOOL IsEncrypted(int Handle)
{
   char Sig[SIG_LEN];

   read(Handle, Sig, SIG_LEN);
   /* Point to beginning of file */
   lseek(Handle, 0, SEEK_SET);

   /* Check for our signature at the beginning of the file */
   return (memcmp(Sig, FileSig, SIG_LEN) == 0) ? TRUE : FALSE;
}

int EncryptFile(int Handle, long FileLen, BYTE PINLen, LPBYTE PIN)
{
   int       ErrCode = 0;
   long      NewLen;
   BYTE      KeyLen;
   CRYPTFILE CFile;

   /* This buffer needs to be 2 blocks larger than file data */
   CFile.FileData = (LPBYTE) calloc(FileLen + (BLOCK_LEN<<1), 1);

   if (CFile.FileData != NULL)
   {
      lseek(Handle, 0, SEEK_SET);

      if (read(Handle, CFile.FileData, FileLen) != -1)
      {
         /* Plain copy of signature */
         memcpy(CFile.Sig, FileSig, SIG_LEN);

         ErrCode = InitializeCiB(1, DeviceNames);

         if (!ErrCode)
         {
            NewLen = FileLen;
            /* Encrypt file and key */
            ErrCode = EncryptMessageSamePub(&NewLen, (LPBYTE) CFile.FileData, 
                                            &KeyLen, CFile.Key, PINLen, PIN);
    
            if (!ErrCode)
            {
               lseek(Handle, 0, SEEK_SET);
               /* Write out new file data */ 
               if ((write(Handle, CFile.Sig, SIG_LEN) == -1)    || 
                   (write(Handle, CFile.Key, KeyLen) == -1)     || 
                   (write(Handle, CFile.FileData, NewLen) == -1))
               {
                  printf("Error writing file\n");
                  ErrCode = -1;
               }
            }
         }
      }
      else
      {
         printf("Error reading file\n");
         ErrCode = -1;
      } 
   }
   else
   {
      printf("Error, insufficient memory\n");
      ErrCode = -1;
   }

   free(CFile.FileData);
   return ErrCode;
}

int DecryptFile(int Handle, long FileLen, BYTE PINLen, LPBYTE PIN) 
{
   int       ErrCode = 0;
   long      NewLen;
   CRYPTFILE CFile;

   CFile.FileData = (LPBYTE) calloc(FileLen+(BLOCK_LEN<<1), sizeof(BYTE));

   if (CFile.FileData != NULL)
   {
      lseek(Handle, 0, SEEK_SET);

      if ((read(Handle, CFile.Sig, SIG_LEN) != -1)     &&
          (read(Handle, CFile.Key, MOD_LEN) != -1)     &&
          (read(Handle, CFile.FileData, FileLen-MOD_LEN-SIG_LEN) != -1))
      {

         ErrCode = InitializeCiB(1, DeviceNames);

         if (!ErrCode)
         {
            /* Length of encrypted message */
            NewLen = FileLen - MOD_LEN - SIG_LEN;
            /* Decrypt key and then file */ 
            ErrCode = DecryptMessage(&NewLen, CFile.FileData, MOD_LEN, 
                                     CFile.Key, PINLen, PIN);
    
            if (!ErrCode)
            {
               lseek(Handle, 0, SEEK_SET);
               /* Restore original file data */
               if (write(Handle, CFile.FileData, NewLen) == -1)
               {
                  printf("Error writing file\n");
                  ErrCode = -1;
               }
               else
                  ftruncate(Handle, NewLen);
            }
         }
      }
      else
      {
         printf("Error reading file\n");
         ErrCode = -1;
      }
   }
   else
   {
      printf("Error, insufficient memory\n");
      ErrCode = -1;
   }

   free(CFile.FileData);
   return ErrCode;
}

int main(int argc, char *argv[])
{
   struct stat Buf;
   long        FileLen;
   char        FileName[40], NewFileName[40], GPIN[40];
   char        *ExtPtr;
   int         FileDes, ErrCode;

   ErrCode = 0;
   if (argc == 1)
   {
      printf("\nUsage:cibcrypt filename\n");
      ErrCode = -1;
   }
   else
   {
      strcpy(FileName, argv[1]);
      FileDes = open(FileName, O_RDWR); 

      if (FileDes == -1)
      {
         printf("Error opening file:%s\n", FileName); 
         ErrCode = -1;
      }
      else
      {
         fstat(FileDes, &Buf);
         FileLen = Buf.st_size;
   
         /* See if the file claims to be encrypted */
         if (!IsEncrypted(FileDes))
         {
            /* Assume group has no PIN */
            GPIN[0] = 0; 

            while ((ErrCode = EncryptFile(FileDes, 
                                          FileLen, 
                                          strlen(GPIN), 
                                          GPIN)) == ERR_BAD_GROUP_PIN)
            {
               printf("\nEnter group PIN :");
               scanf("%s", GPIN);
            }

            if (!ErrCode)
            {
               strcpy(NewFileName, FileName);
               /* Append new extension */
               strcat(NewFileName, ".CiB");
               if (rename(FileName, NewFileName) == -1)
               {
                  printf("Error renaming file %s\n", FileName);
                  ErrCode = -1;
               }
            }
            else
               printf("\nError encrypting file %s <0x%4.4X>\n",FileName,ErrCode);
         }
         else 
         {
            /* Assume group has no PIN */
            GPIN[0] = 0; 

            while ((ErrCode = DecryptFile(FileDes, 
                                          FileLen, 
                                          strlen(GPIN), 
                                          GPIN)) == ERR_BAD_GROUP_PIN)
            {
               printf("\nEnter group PIN :");
               scanf("%s", GPIN);
            }

            if (!ErrCode)
            {
               strcpy(NewFileName, FileName);
               ExtPtr = strstr(NewFileName, ".CiB");
               if (ExtPtr != NULL)
               {
                  *ExtPtr = 0;
                  if (rename(FileName, NewFileName) == -1)
                  {
                     printf("Error renaming file %s\n", FileName);
                     ErrCode = -1;
                  }
               }
            }
            else
               printf("Error decrypting file %s\n", FileName);
         }
      }
   }

   close(FileDes);
   return ErrCode;
}

